;********************************************************************
; ALIEN BEHAVIOR MODULE                                             *
;********************************************************************
;********************************************************************
; Jump table handler:   BehaviorUpdate
;********************************************************************
BehaviorUpdate
                ldy alienState
                lda alienStateJumpTableLow,y
                sta ALIENBEHAVIORL
                lda alienStateJumpTableHigh,y
                sta ALIENBEHAVIORH
                jmp (ALIENBEHAVIORL)

;********************************************************************
; Subroutine:   TreatAlienStatePickup
;********************************************************************
TreatAlienStatePickup
                ;****************************
                ;* alienState:              *
                ;*  ALIEN_STATE_PICKUP      *
                ;*                          *
                ;****************************
                ldy #TIMER_C
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatalienstatepickup_nopickup
                jsr ResetTimer
                lda pickupCounter
                cmp #$00
                beq .treatalienstatepickup_check
                jsr StartTimer
                ;do pick up task ************************************
                jsr PickupAliens
                dec pickupCounter
                jsr AssignAttackPatternTable
.treatalienstatepickup_nopickup
                jsr AttackMovement
                rts
.treatalienstatepickup_check
                lda #ALIEN_STATE_GRAPPER
                sta alienState
                ldy #TIMER_C
                jsr RestartTimer
                rts

;********************************************************************
; Subroutine:   TreatAlienStateGrapper
;********************************************************************
TreatAlienStateGrapper
                ;****************************
                ;* alienState:              *
                ;*  ALIEN_STATE_GRAPPER     *
                ;*                          *
                ;****************************
                ldy #TIMER_C
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatalienstategrapper_nograpper
                jsr RestartTimer
                ;do grapper task ************************************
                jsr GrapperToEscape
                ;check if all aliens are type CONVOY_MODE
                jsr CheckAllConvoyMode
                cmp #TRUE
                bne .treatalienstategrapper_nograpper
                ;get a random table
                lda #ANGRY_TABLES_MAX
                jsr GetRandomValueWithInterval
                stx ANGRYTABLEINDEX
                ;assign all aliens to the same table
                jsr AssignAngryPatternTable
                ;then change state to ALIEN_STATE_ANGRY
                lda #ALIEN_STATE_ANGRY
                sta alienState
                ldy #TIMER_D
                jsr RestartTimer
                rts
.treatalienstategrapper_nograpper
                jsr AttackMovement
                rts

;********************************************************************
; Subroutine:   TreatAlienStateAngry
;********************************************************************
TreatAlienStateAngry
                ;****************************
                ;* alienState:              *
                ;*  ALIEN_STATE_ANGRY       *
                ;*                          *
                ;****************************
                ldy #TIMER_D
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatalienstateangry_exit
                jsr RestartTimer
                ;do angry task **************************************
                jsr AngryMovement
                ;check if move is finished
                jsr CheckAllConvoyMode
                cmp #TRUE
                bne .treatalienstateangry_exit
                lda DIFFICULTY
                cmp #$04
                beq .treatalienstateangry_skip
                inc DIFFICULTY
.treatalienstateangry_skip
                inc PROXIMITY
                lda PROXIMITY
                cmp #$04
                bne .treatalienstateangry_skip2
                ;reset convoy to initial position
                ;if angry convoy is landed
                lda #$00
                sta PROXIMITY
                jsr AliensIngameInit
.treatalienstateangry_skip2
                ;init pickup counter
                lda #NR_OF_PICKUP_TASKS
                sta pickupCounter
                lda #ALIEN_STATE_PICKUP
                sta alienState
                ldy #TIMER_C
                jsr RestartTimer
.treatalienstateangry_exit
                rts

;********************************************************************
; Subroutine:   PickupAliens
; Depending on DIFFICULTY, pick up one or more active, random aliens
; which are currently in convoy mode.
;********************************************************************
PickupAliens
                lda DIFFICULTY
                sta REPEATCOUNTER
                ;copy fleetArray to alienTempArray
                ldy #$00
.pickupaliens_copyloop
                lda fleetArray,Y
                sta alienTempArray,Y
                iny
                cpy #ALIENS_MAX
                bne .pickupaliens_copyloop
.pickupaliens_loopDIFFICULTY
                ;create a list of current active convoy aliens (index numbers)
                ;and get the size of attackCandidatesList array.
                ldx #$00
                ldy #$00
.pickupaliens_loopallaliens
                lda alienTempArray,X
                cmp #TRUE                   ;get the active ones
                bne .pickupaliens_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE            ;get the convoy ones
                bne .pickupaliens_skip
                txa                         ;store the index number
                sta attackCandidatesList,Y
                iny
.pickupaliens_skip
                inx
                cpx #ALIENS_MAX             ;for all aliens
                bne .pickupaliens_loopallaliens
                sty sizeOfAttackCandidatesList
                ;get random value between 0...sizeOfAttackCandidatesList
                lda sizeOfAttackCandidatesList
                jsr GetRandomValueWithInterval
                ;get one element from attackCandidatesList
                lda attackCandidatesList,X
                ;set the element at alienPickupArray
                tax
                lda #TRUE
                sta alienPickupArray,X
                ;remove element from alienTempArray
                lda #FALSE
                sta alienTempArray,X
                ;dep. on DIFFICULTY, repeat random pickup
                dec REPEATCOUNTER
                lda REPEATCOUNTER
                cmp #$00
                bne .pickupaliens_loopDIFFICULTY
                rts

;********************************************************************
; Subroutine:   AssignAttackPatternTable
; Change the alien mode from CONVOY_MODE to ATTACK_MODE and
; remove (convoy) alien from screen,
; Get a random attack pattern table and 
; assign the alien movement pointer.
; Repeat that for all picked up aliens. 
;********************************************************************
AssignAttackPatternTable
                lda #$00
                sta assignIndex
.assignattackpatterntable_loopallaliens
                ldy assignIndex
                lda alienPickupArray,Y
                cmp #TRUE
                bne .assignattackpatterntable_skip
                lda alienModeArray,Y
                cmp #CONVOY_MODE            ;get the convoy ones
                bne .assignattackpatterntable_skip
                ;change mode
                lda #ATTACK_MODE
                sta alienModeArray,Y
                ;delete pickup array
                lda #FALSE
                sta alienPickupArray,Y
                ;remove (convoy) alien from screen
                lda fleetXposArray,Y
                sta CURRENTSHAPEXPOS
                lda fleetYposArray,Y
                sta CURRENTSHAPEYPOS
                ;delete alien from screen
                lda #<clrAlien
                sta STRINGL
                lda #>clrAlien
                sta STRINGH
                lda #<clrAlienColor
                sta TEXTCOLORL
                lda #>clrAlienColor
                sta TEXTCOLORH
                lda #$01
                sta BLOCKWIDTH
                lda #$02 
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
                ;get a random attack pattern table (0...ATTACK_TABLES_MAX)
                lda #ATTACK_TABLES_MAX
                jsr GetRandomValueWithInterval
                ;assign the alien movement pointer
                lda tocAttackPatternTableLow,X
                pha
                lda tocAttackPatternTableHigh,X
                pha
                lda assignIndex
                asl
                tax
                pla
                sta ATTACKPOINTER+1,X
                pla
                sta ATTACKPOINTER,X
.assignattackpatterntable_skip
                inc assignIndex
                lda assignIndex
                cmp #ALIENS_MAX             ;for all picked up aliens
                bne .assignattackpatterntable_loopallaliens
                rts

;********************************************************************
; Subroutine:   AttackMovement
;********************************************************************
AttackMovement
                ldy #TIMER_D
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .attackmovement_notask
                jsr RestartTimer
                ;do task ********************************************
                lda #$00
                sta FLEETINDEX
.attackmovement_loopallaliens
                ldx FLEETINDEX
                lda fleetArray,X
                cmp #TRUE                   ;get the active ones
                bne .attackmovement_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE            ;get the != convoy ones
                beq .attackmovement_skip
                lda alienModeArray,X
                cmp #GRAPPER_MODE           ;get the != grapper ones
                beq .attackmovement_skip
                ;set the current alien number for DrawShape
                lda FLEETINDEX
                sta CURRENTALIENNUMBER
                asl
                tax
                ;get the assigned movement pointer for current alien number
                lda ATTACKPOINTER,X
                sta ATTACKPATTERNL
                lda ATTACKPOINTER+1,X
                sta ATTACKPATTERNH
                ;get the command from table and execute
                jsr GetAttackPatternCommand
                jsr AlienCommandCase
.attackmovement_skip
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .attackmovement_loopallaliens
.attackmovement_notask
                rts

;********************************************************************
; Subroutine:   GrapperToEscape
;********************************************************************
GrapperToEscape
                ldx #$00
.grappertoescape_loop
                lda fleetArray,X
                cmp #TRUE                   ;get the active ones
                bne .grappertoescape_skip
                lda alienModeArray,X
                cmp #GRAPPER_MODE            ;get the grapper
                bne .grappertoescape_skip
                ;convert to escape if grapp time over
                lda alienGrappTimeArray,X
                cmp #$00
                bne .grappertoescape_skip
                lda #ESCAPE_MODE
                sta alienModeArray,X
.grappertoescape_skip
                inx
                cpx #ALIENS_MAX
                bne .grappertoescape_loop
                rts

;********************************************************************
; Subroutine:   AssignAngryPatternTable
; Change the alien mode from CONVOY_MODE to ATTACK_MODE and
; remove (convoy) alien from screen,
; assign the angry movement pointer.
; Repeat that for all aliens.
;********************************************************************
AssignAngryPatternTable
                lda #$00
                sta assignIndex
.assignangrypatterntable_loopallaliens
                ldy assignIndex
                ;change mode
                lda #ATTACK_MODE
                sta alienModeArray,Y
                ;remove (convoy) alien from screen
                lda fleetXposArray,Y
                sta CURRENTSHAPEXPOS
                lda fleetYposArray,Y
                sta CURRENTSHAPEYPOS
                ;delete alien from screen
                lda #<clrAlien
                sta STRINGL
                lda #>clrAlien
                sta STRINGH
                lda #<clrAlienColor
                sta TEXTCOLORL
                lda #>clrAlienColor
                sta TEXTCOLORH
                lda #$01
                sta BLOCKWIDTH
                lda #$02 
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
                ;assign the angry movement table
                ldx ANGRYTABLEINDEX
                lda tocAngryPatternTableLow,X
                pha
                lda tocAngryPatternTableHigh,X
                pha
                lda assignIndex
                asl
                tax
                pla
                sta ATTACKPOINTER+1,X
                pla
                sta ATTACKPOINTER,X
                inc assignIndex
                lda assignIndex
                cmp #ALIENS_MAX             ;for all aliens
                bne .assignangrypatterntable_loopallaliens
                rts

;********************************************************************
; Subroutine:   AngryMovement
;********************************************************************
AngryMovement
                lda #$00
                sta FLEETINDEX
.angrymovement_loopallaliens
                ldx FLEETINDEX
                lda fleetArray,X
                cmp #TRUE                   ;get the active ones
                bne .angrymovement_skip
                ;set the current alien number for DrawShape
                lda FLEETINDEX
                sta CURRENTALIENNUMBER
                asl
                tax
                ;get the assigned movement pointer for current alien number
                lda ATTACKPOINTER,X
                sta ATTACKPATTERNL
                lda ATTACKPOINTER+1,X
                sta ATTACKPATTERNH
                ;get the command from table and execute
                jsr GetAttackPatternCommand
                jsr AlienCommandCase
.angrymovement_skip
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .angrymovement_loopallaliens
                rts

;********************************************************************
; Subroutine:   CheckAllConvoyMode
;********************************************************************
CheckAllConvoyMode
                ldx #$00
.checkallconvoymode_loop
                lda fleetArray,X
                cmp #TRUE                   ;get the active ones
                bne .checkallconvoymode_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE
                bne .checkallconvoymode_exit
.checkallconvoymode_skip
                inx
                cpx #ALIENS_MAX
                bne .checkallconvoymode_loop
                lda #TRUE
                rts
.checkallconvoymode_exit
                lda #FALSE
                rts

;********************************************************************
; Subroutine:   DoCommandAttack
;********************************************************************
DoCommandAttack
                ;delete old shape
                jsr DeleteOldAttackAlienFromScreen
                ;get shape data from table
                jsr GetAttackPatternShapeType
                jsr SetAlienShapeData
                ;get X position from table
                jsr GetAttackPatternXoffset
                jsr AddOffsetToXpos
                ;get Y position from table
                jsr GetAttackPatternYoffset
                jsr AddOffsetToYpos
                ;draw shape
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShapeCollisionCheck
                ;update position array for current alien
                ldx CURRENTALIENNUMBER
                lda CURRENTSHAPEXPOS
                sta fleetXposArray,X
                lda CURRENTSHAPEYPOS
                sta fleetYposArray,X
                ;swap position if not angry and leaving visible Y
                lda alienState
                cmp #ALIEN_STATE_ANGRY
                beq .docommandattack_swapend
                lda CURRENTSHAPEYPOS
                cmp #$FE
                bne .docommandattack_skiplow
                lda #SCREEN_HIGHT-1
                sta CURRENTSHAPEYPOS
                sta fleetYposArray,X
                jmp .docommandattack_swapend
.docommandattack_skiplow
                lda CURRENTSHAPEYPOS
                cmp #SCREEN_HIGHT
                bne .docommandattack_swapend
                lda #$01
                sta CURRENTSHAPEYPOS
                sta fleetYposArray,X
.docommandattack_swapend
                ;save the current position
                ldx CURRENTALIENNUMBER
                lda fleetXposArray,X
                sta fleetOldXposArray,X
                lda fleetYposArray,X
                sta fleetOldYposArray,X
                ;save shadow position if state: ALIEN_STATE_ANGRY
                lda alienState
                cmp #ALIEN_STATE_ANGRY
                bne .docommandattack_skip
                lda fleetXposArray,X
                sta shadowXposArray,X
                lda fleetYposArray,X
                sta shadowYposArray,X
.docommandattack_skip
                ;save the current shape type
                lda CURRENTSHAPENUMBER
                sta alienShapeTypeArray,X
                ;update movement pointer to the next table entry
                jsr IncMovementPointer
                rts

;********************************************************************
; Subroutine:   DoCommandConvoy
;********************************************************************
DoCommandConvoy
                ;delete old shape
                jsr DeleteOldAttackAlienFromScreen
                ;restore with starting position
                ldx CURRENTALIENNUMBER
                lda shadowXposArray,X
                sta fleetXposArray,X
                lda shadowYposArray,X
                sta fleetYposArray,X
                ;change mode for current alien to CONVOY
                lda #CONVOY_MODE
                sta alienModeArray,X
                rts

;********************************************************************
; Subroutine:   DoCommandGrapper
;********************************************************************
DoCommandGrapper
                ;delete old shape
                jsr DeleteOldAttackAlienFromScreen
                ;update position array for current alien
                ;jsr UpdatePositionArray
                ldx CURRENTALIENNUMBER
                lda CURRENTSHAPEXPOS
                sta fleetXposArray,X
                lda CURRENTSHAPEYPOS
                sta fleetYposArray,X
                ;change mode for current alien to GRAPPER
                lda #GRAPPER_MODE
                sta alienModeArray,X
                ;get and store grapp time
                jsr GetAttackPatternShapeType
                lda CURRENTSHAPENUMBER
                sta alienGrappTimeArray,X
                ;update movement pointer to the next table entry
                jsr IncMovementPointer
                rts

;********************************************************************
; Subroutine:   UpdatePositionArray
;********************************************************************
UpdatePositionArray
                ;update position array for current alien
                ldx CURRENTALIENNUMBER
                lda CURRENTSHAPEXPOS
                sta fleetXposArray,X
                lda CURRENTSHAPEYPOS
                sta fleetYposArray,X
                rts

;********************************************************************
; Subroutine:   SetAlienShapeData
; Input:        CURRENTSHAPENUMBER: alien type (ALIEN_TYPE_00..33)
;********************************************************************
SetAlienShapeData
                ldx CURRENTSHAPENUMBER
                lda alienShapeAddrTableLow,X
                sta STRINGL
                lda alienShapeAddrTableHigh,X
                sta STRINGH
                lda alienShapeColorAddrTableLow,X
                sta TEXTCOLORL
                lda alienShapeColorAddrTableHigh,X
                sta TEXTCOLORH
                txa
                asl
                tax
                lda alienSizeTable,X
                sta BLOCKWIDTH
                lda alienSizeTable+1,X
                sta BLOCKHEIGHT
                rts

;********************************************************************
; Subroutine:   AddOffsetToXpos
; Input:        CURRENTSHAPEXOFFSET:
;               CURRENTALIENNUMBER:         alien number
;               fleetOldXposArray:
; Output:       CURRENTSHAPEXPOS:
;********************************************************************
AddOffsetToXpos
                ldx CURRENTALIENNUMBER
                clc
                lda fleetXposArray,X
                clc
                adc CURRENTSHAPEXOFFSET
                sta CURRENTSHAPEXPOS
                rts

;********************************************************************
; Subroutine:   AddOffsetToYpos
; Input:        CURRENTSHAPEYOFFSET:
;               CURRENTALIENNUMBER:         alien number
;               aliensInitialYArray:
; Output:       CURRENTSHAPEYPOS:
;********************************************************************
AddOffsetToYpos
                ldx CURRENTALIENNUMBER
                clc
                lda fleetYposArray,X
                clc
                adc CURRENTSHAPEYOFFSET
                sta CURRENTSHAPEYPOS
                rts

;********************************************************************
; Subroutine:   GetAttackPatternCommand
; Input:        ATTACKPATTERNL:
; Output:       CURRENTALIENCOMMAND:                alien mode
;********************************************************************
GetAttackPatternCommand
                ldy #$00
                lda (ATTACKPATTERNL),Y
                sta CURRENTALIENCOMMAND
                rts

;********************************************************************
; Subroutine:   GetAttackPatternShapeType
; Input:        ATTACKPATTERNL:        
; Output:       CURRENTSHAPENUMBER:         alien shape number
;********************************************************************
GetAttackPatternShapeType
                ldy #$01
                lda (ATTACKPATTERNL),Y
                sta CURRENTSHAPENUMBER
                rts

;********************************************************************
; Subroutine:   GetAttackPatternXoffset
; Input:        ATTACKPATTERNL:          
; Output:       CURRENTSHAPEXOFFSET:        delta to x position
;********************************************************************
GetAttackPatternXoffset
                ldy #$02
                lda (ATTACKPATTERNL),Y
                sta CURRENTSHAPEXOFFSET
                rts
                
;********************************************************************
; Subroutine:   GetAttackPatternYoffset
; Input:        ATTACKPATTERNL:         
; Output:       CURRENTSHAPEYOFFSET:        delta to y position
;********************************************************************
GetAttackPatternYoffset
                ldy #$03
                lda (ATTACKPATTERNL),Y
                sta CURRENTSHAPEYOFFSET
                rts

;********************************************************************
; Subroutine:   IncMovementPointer
;********************************************************************
IncMovementPointer
;update movement pointer to the next table entry
                lda CURRENTALIENNUMBER
                asl
                tax
                clc
                lda ATTACKPOINTER,X
                adc #$04                ;number of fields
                sta ATTACKPOINTER,X
                lda ATTACKPOINTER+1,X
                adc #$00
                sta ATTACKPOINTER+1,X
                rts

;********************************************************************
; Subroutine:   DeleteOldAttackAlienFromScreen
;********************************************************************
DeleteOldAttackAlienFromScreen
                ldx CURRENTALIENNUMBER
                lda fleetOldXposArray,X
                sta CURRENTSHAPEXPOS
                lda fleetOldYposArray,X
                sta CURRENTSHAPEYPOS
                ldx #ALIEN_SHAPE_00
                stx CURRENTSHAPENUMBER
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr SetAlienShapeData
                jsr DrawShape
                rts

;********************************************************************
; Subroutine:   DeleteAllAliensFromScreen
;********************************************************************
DeleteAllAliensFromScreen
                lda #$00
                sta FLEETINDEX
.deleteallaliensfromscreen_next
                ldx FLEETINDEX
                lda fleetArray,X
                cmp #PRESENT
                bne .deleteallaliensfromscreen_skip
                lda fleetOldXposArray,X
                sta CURRENTSHAPEXPOS
                lda fleetOldYposArray,X
                sta CURRENTSHAPEYPOS
                ldx #ALIEN_SHAPE_00
                stx CURRENTSHAPENUMBER
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr SetAlienShapeData
                jsr DrawShape
.deleteallaliensfromscreen_skip
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .deleteallaliensfromscreen_next
                rts

;********************************************************************
; Subroutine:   AlienCommandCase
;********************************************************************
AlienCommandCase
;if CMD_ATTACK
                lda CURRENTALIENCOMMAND
                cmp #CMD_ATTACK
                bne .aliencommandcase_shoot
                jsr DoCommandAttack
                jmp .aliencommandcase_skip
;if CMD_ATTACK_SHOOT
.aliencommandcase_shoot
                lda CURRENTALIENCOMMAND
                cmp #CMD_ATTACK_SHOOT
                bne .aliencommandcase_grapper
                jsr DoCommandAttack
                jsr AlienBulletsFire
                jmp .aliencommandcase_skip
;if CMD_GRAPPER
.aliencommandcase_grapper
                lda CURRENTALIENCOMMAND
                cmp #CMD_GRAPPER
                bne .aliencommandcase_convoy
                jsr DoCommandGrapper
                jmp .aliencommandcase_skip
;if CMD_CONVOY
.aliencommandcase_convoy
                lda CURRENTALIENCOMMAND
                cmp #CMD_CONVOY
                bne .aliencommandcase_skip
                jsr DoCommandConvoy
.aliencommandcase_skip
                rts

;********************************************************************
;Alien behavior data ************************************************
;********************************************************************
alienState                          ;the state of the alien behavior
                !byte ALIEN_STATE_PICKUP

;Jump table for the alien behavior
alienStateJumpTableLow
                !byte <TreatAlienStatePickup
                !byte <TreatAlienStateGrapper
                !byte <TreatAlienStateAngry

alienStateJumpTableHigh
                !byte >TreatAlienStatePickup
                !byte >TreatAlienStateGrapper
                !byte >TreatAlienStateAngry

;used for the random pick up routines
alienTempArray
                !fill ALIENS_MAX,0          ;copy of: fleetArray

attackCandidatesList
                !fill ALIENS_MAX,0          ;number list of active convoy aliens

sizeOfAttackCandidatesList
                !byte 0                     ;number of active convoy aliens

alienPickupArray
                !fill ALIENS_MAX,0          ;binary array of aliens for the next attack phase

assignIndex
                !byte 0                     ;temp loop counter for AssignAttackPatternTable

pickupCounter
                !byte NR_OF_PICKUP_TASKS

;grapper time
alienGrappTimeArray
                !fill ALIENS_MAX,0

;attack pattern tables: command, currentShapeNumber, currentShapeXoffset, currentShapeYoffset
attackPattern1Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_27, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_17,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                ;--------------------------------------------
                !byte CMD_GRAPPER,      $05,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 0   ;right
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 1   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 0   ;right
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 1   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 0   ;right
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_32,  0, 2   ;up
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0

                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_27,  0,-1   ;roll left
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_24,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_26,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_00,  0,-1
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  0, 0
                ;--------------------------------------------

attackPattern2Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_19,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_18,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_17,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0,-1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  0, 0   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 0   ;right
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 1   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0   ;up
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  1, 0
                ;--------------------------------------------

attackPattern3Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_18,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_19,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_17,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                ;--------------------------------------------
                !byte CMD_GRAPPER,      $03,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_18,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_19,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_17,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  1, 0
                ;--------------------------------------------

attackPattern4Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_14, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_15,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_19,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_17,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  0, 1   ;right
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 1   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 0   ;right
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_32,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_34,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_35,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_36,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 1   ;left
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_33,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_37,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_38,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_39,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0   ;up
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  1, 0
                ;--------------------------------------------

attackPattern5Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_07, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  1, 0
                ;--------------------------------------------

;table of contents for attack pattern tables
tocAttackPatternTableLow
                !byte <attackPattern1Table
                !byte <attackPattern2Table
                !byte <attackPattern3Table
                !byte <attackPattern4Table
                !byte <attackPattern5Table

tocAttackPatternTableHigh
                !byte >attackPattern1Table
                !byte >attackPattern2Table
                !byte >attackPattern3Table
                !byte >attackPattern4Table
                !byte >attackPattern5Table

angryPattern1Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                ;--------------------------------------------
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_26,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  1, 1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_28,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_25, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_31, -1, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_24, -1,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_29,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_27,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  0, 0
                ;--------------------------------------------

angryPattern2Table
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0   ;down
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0,-1   ;left
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_19,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_18,  0,-1
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_17,  0,-1
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_22, -1, 0   ;down
                !byte CMD_ATTACK,       ALIEN_SHAPE_06,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_21, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_07,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_20, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_08,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_23, -1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 0
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_17,  0, 0   ;right
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_06,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_18,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_07,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_19,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_08,  0, 1
                !byte CMD_ATTACK,       ALIEN_SHAPE_16,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  0, 1
                ;--------------------------------------------
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0   ;up
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK_SHOOT, ALIEN_SHAPE_05,  1, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_30,  0, 0
                !byte CMD_ATTACK,       ALIEN_SHAPE_05,  1, 0
                ;--------------------------------------------
                !byte CMD_CONVOY,       ALIEN_SHAPE_00,  0, 0
                ;--------------------------------------------

tocAngryPatternTableLow
                !byte <angryPattern1Table
                !byte <angryPattern2Table

tocAngryPatternTableHigh
                !byte >angryPattern1Table
                !byte >angryPattern2Table

;********************************************************************
; ALIEN BEHAVIOR MODULE END                                         *
;********************************************************************