;********************************************************************
; MOTHERSHIP MODULE                                                 *
;********************************************************************
;********************************************************************
; Jump table handler:   MothershipUpdate
;********************************************************************
MothershipUpdate
                ldy bossState
                lda bossStateJumpTableLow,y
                sta BOSSBEHAVIORL
                lda bossStateJumpTableHigh,y
                sta BOSSBEHAVIORH
                jmp (BOSSBEHAVIORL)

;********************************************************************
; Subroutine:   TreatBossStateActive
;********************************************************************
TreatBossStateActive
                ;****************************
                ;* bossState:               *
                ;*  BOSS_STATE_ACTIVE       *
                ;*                          *
                ;****************************
                lda PLAYERACTIVE
                cmp #TRUE
                beq .treatbossstateactive_continue
                rts
.treatbossstateactive_continue
                ;only for wave 3
                lda CURRENTWAVE
                cmp #FLOW_WAVE_3
                beq .treatbossstateactive_continue2
                rts
.treatbossstateactive_continue2
                ;check if boss was hit
                lda bossWasHit
                cmp #TRUE
                bne .treatbossstateactive_cont
                ;change state to: boss was hit
                lda #BOSS_STATE_HIT
                sta bossState
                rts
.treatbossstateactive_cont
                ;draw the assembled mothership
                lda CURRENTBOSSXPOS
                sta CURRENTSHAPEXPOS
                lda initialMothershipYpos
                sta CURRENTSHAPEYPOS
                lda #<mothershipShapeTable
                sta STRINGL
                lda #>mothershipShapeTable
                sta STRINGH
                lda #<mothershipColorTable
                sta TEXTCOLORL
                lda #>mothershipColorTable
                sta TEXTCOLORH
                lda #$0A
                sta BLOCKWIDTH
                lda #$14
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-3
                sta MAXSCREENX
                jsr DrawShape
                ;rotate alien pilot frame
                ldy #TIMER_F
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatbossstateactive_exit
                jsr RestartTimer
                ;do task ********************************************
                inc ALIENPILOTFRAMEINDEX
                lda ALIENPILOTFRAMEINDEX
                cmp #$08
                bne .treatbossstateactive_rotateexit
                lda #$00
                sta ALIENPILOTFRAMEINDEX
.treatbossstateactive_rotateexit
                ;copy alien pilot to mothership
                jsr CopyAlienPilot
                ;update the rotating purple layer
                jsr RotatePurLayer
                ;move mothership further down
                jsr MoveMothership
                ;let the boss fire bullets at 4 Y positions
                jsr BossUpdateFiring
.treatbossstateactive_exit
                rts

;********************************************************************
; Subroutine:   TreatBossStateHit
;********************************************************************
TreatBossStateHit
                ;****************************
                ;* bossState:               *
                ;*  BOSS_STATE_HIT          *
                ;*                          *
                ;****************************
                lda #FALSE
                sta bossWasHit
                sta PLAYERACTIVE
                ;stop mothership movement speed timer
                ldy #TIMER_E
                jsr ResetTimer
                ;stop alien pilot animation speed timer
                ldy #TIMER_F
                jsr ResetTimer
                ;stops in flight bullets from scoring
                jsr BulletsReset
                ;stop starfield background
                lda #FALSE
                sta STARSALLOWED
                clc
                lda CURRENTBOSSXPOS
                adc #$04
                sta particleExplosionX
                lda #$0C
                sta particleExplosionY
                ;trigger explosion animation
                lda #TRUE
                sta particleExplosionTriggered
                ;start animation speed timer
                ldy #TIMER_A
                jsr StartTimer
                ;trigger particle explosion sound
                jsr TriggerSoundParticleExplosion
                ;change state to: explosion
                lda #BOSS_STATE_EXPL
                sta bossState
                rts

;********************************************************************
; Subroutine:   TreatBossStateExp
;********************************************************************
TreatBossStateExp
                ;****************************
                ;* bossState:               *
                ;*  BOSS_STATE_EXPL         *
                ;*                          *
                ;****************************
                ;check for pending explosion
                lda particleExplosionTriggered
                cmp #FALSE
                bne .treatbossstateexp_end
                ;clear screen
                jsr ClearScreen
                jsr DoCommandPrintTopLines
                ;start starfield background
                lda #TRUE
                sta STARSALLOWED
                ;calculate mothership score
                jsr MothershipScoreCalculation
                ;display the score
                ;mid value
                clc
                lda CURRENTBOSSXPOS
                adc #$06
                sta CURRENTXPOS
                lda #$0B
                sta CURRENTYPOS
                lda MOTHERSHIPSCORE
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;low value is 00
                lda #$0D
                sta CURRENTYPOS
                lda #$00
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;add mothership score to top line score
                lda MOTHERSHIPSCORE
                sta VALUETOSCORE
                jsr IncreaseScoreHigh
                ;change state to: score display
                lda #BOSS_STATE_SCORE
                sta bossState
.treatbossstateexp_end
                rts

;********************************************************************
; Subroutine:   TreatBossStateScore
;********************************************************************
TreatBossStateScore
                ;****************************
                ;* bossState:               *
                ;*  BOSS_STATE_SCORE        *
                ;*                          *
                ;****************************
                ;start the play break *******************************
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .treatbossstatescore_playbreak
                jsr StartTimer
                rts
.treatbossstatescore_playbreak
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatbossstatescore_end
                jsr ResetTimer
                ;end of play break **********************************
                ;change state to: active
                lda #BOSS_STATE_ACTIVE
                sta bossState
                ;play tune 'Fuer Elise'
                jsr TriggerSoundTune3
                ;change flow state to: the end
                lda #FLOW_STATE_THEEND
                sta FLOWSTATE
.treatbossstatescore_end
                rts

;********************************************************************
; Subroutine:   CopyAlienPilot
;********************************************************************
CopyAlienPilot
                ldy ALIENPILOTFRAMEINDEX
                lda alienPilotFrameTableLow,Y
                sta ALIENPILOTFRAMEL
                lda alienPilotFrameTableHigh,Y
                sta ALIENPILOTFRAMEH
                ldy #$00
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+95
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+96
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+97
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+98
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+105
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+106
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+107
                iny
                lda (ALIENPILOTFRAMEL),Y
                sta mothershipShapeTable+108
                rts

;********************************************************************
; Subroutine:   RotatePurLayer
;********************************************************************
RotatePurLayer
                ;slow down and sync
                lda ALIENPILOTFRAMEINDEX
                beq .rotatepurlayer_rotate
                rts
.rotatepurlayer_rotate
                lda mothershipShapeTable+184
                sta rotatingLayerTemp
                lda mothershipShapeTable+174
                sta mothershipShapeTable+184
                lda mothershipShapeTable+164
                sta mothershipShapeTable+174
                lda mothershipShapeTable+154
                sta mothershipShapeTable+164
                lda mothershipShapeTable+144
                sta mothershipShapeTable+154
                lda mothershipShapeTable+134
                sta mothershipShapeTable+144
                lda mothershipShapeTable+124
                sta mothershipShapeTable+134
                lda mothershipShapeTable+114
                sta mothershipShapeTable+124
                lda mothershipShapeTable+104
                sta mothershipShapeTable+114
                lda mothershipShapeTable+94
                sta mothershipShapeTable+104
                lda mothershipShapeTable+84
                sta mothershipShapeTable+94
                lda mothershipShapeTable+74
                sta mothershipShapeTable+84
                lda mothershipShapeTable+64
                sta mothershipShapeTable+74
                lda mothershipShapeTable+54
                sta mothershipShapeTable+64
                lda mothershipShapeTable+44
                sta mothershipShapeTable+54
                lda mothershipShapeTable+34
                sta mothershipShapeTable+44
                lda mothershipShapeTable+24
                sta mothershipShapeTable+34
                lda mothershipShapeTable+14
                sta mothershipShapeTable+24
                lda rotatingLayerTemp
                sta mothershipShapeTable+14
                ;rotate x offset array
                lda xOffsetPurLayerArray+17
                sta xOffsetTemp
                ldx #SIZE_OF_PURPLE_LAYER-1
                ldy #SIZE_OF_PURPLE_LAYER
.rotatepurlayer_loop
                lda xOffsetPurLayerArray,X
                sta xOffsetPurLayerArray,Y
                dex
                dey
                bne .rotatepurlayer_loop
                lda xOffsetTemp
                sta xOffsetPurLayerArray
                rts

;********************************************************************
; Subroutine:   MoveMothership
;********************************************************************
MoveMothership
                ldy #TIMER_E
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .movemothership_notask
                jsr RestartTimer
                ;do task ********************************************
                ;trigger alert sound, all the time
                jsr TriggerSoundAlert
                ;stop movement at low position
                lda CURRENTBOSSXPOS
                cmp #$08
                beq .movemothership_notask
                dec CURRENTBOSSXPOS
                ;adjust currentBunkersEdgeXArray
                ldx #$00
.movemothership_loop
                dec currentBunkersEdgeXArray,X
                inx
                cpx #NO_OF_BUNKER_EDGES
                bne .movemothership_loop
.movemothership_notask
                rts

;********************************************************************
; Subroutine:   MothershipBulletCollisions
;               Called during raster interrupt
;********************************************************************
MothershipBulletCollisions
                ;check for collided bullets
                ldx CURRENTBUNKERSEDGEINDEX
                lda currentBunkersEdgeXArray,X
                sta BULLETSXCHARCOL
                lda currentBunkersEdgeYArray,X
                sta BULLETSYCHARCOL
                ;adjust bunkers edge X pos if bunker destroyed
                tay
                lda bunkerFrameIndexTable,Y
                cmp #BUNKER_FRAME_INDEX_10
                bne .mothershipbulletcollisions_skip
                clc
                lda currentBunkersEdgeXArray,X
                adc xOffsetArray,Y
                sta BULLETSXCHARCOL
.mothershipbulletcollisions_skip
                lda #TRUE
                sta BULLETSDIRCOL
                jsr BulletsCollided
                bne .mothershipbulletcollisions_collision
                rts
.mothershipbulletcollisions_collision
                ;get hit character
                ldy BULLETSYCHARCOL
                lda screenRAMRowLow,Y
                sta BULLETSCREENL
                lda screenRAMRowHigh,Y
                sta BULLETSCREENH
                ldy BULLETSXCHARCOL
                lda (BULLETSCREENL),Y
                sta HITCHARACTER
                ;check if bunker was hit
                lda HITCHARACTER
                and #%11111100  ;00-03
                cmp #CHRGROUP_DAMAGE
                bne .mothershipbulletcollisions_skip1
                ;get buffer address of bunker
                ldy BULLETSYCHARCOL
                lda bunkerAddrTableLow,Y
                sta BUNKERADDRESSL
                lda bunkerAddrTableHigh,Y
                sta BUNKERADDRESSH
                ;increase damage
                ldx BULLETSYCHARCOL
                inc bunkerFrameIndexTable,X
                ;get damage frame address
                lda bunkerFrameIndexTable,X
                tax
                lda bunkerDamageFrameTableLow,X
                sta BUNKERDAMAGEFRAMEL
                lda bunkerDamageFrameTableHigh,X
                sta BUNKERDAMAGEFRAMEH
                ;copy damage data to buffer
                ldy #$00
                lda (BUNKERDAMAGEFRAMEL),Y
                sta (BUNKERADDRESSL),Y
                iny
                lda (BUNKERDAMAGEFRAMEL),Y
                sta (BUNKERADDRESSL),Y
                iny
                lda (BUNKERDAMAGEFRAMEL),Y
                sta (BUNKERADDRESSL),Y
                iny
                lda (BUNKERDAMAGEFRAMEL),Y
                sta (BUNKERADDRESSL),Y
                ;update bunkers edge X position
                iny
                ldx CURRENTBUNKERSEDGEINDEX
                clc
                lda currentBunkersEdgeXArray,X
                adc (BUNKERDAMAGEFRAMEL),Y
                sta currentBunkersEdgeXArray,X
                rts
.mothershipbulletcollisions_skip1
                ;check if protection bar was hit
                lda HITCHARACTER
                and #%11110000  ;00-0E
                cmp #CHRGROUP_PROTBAR
                bne .mothershipbulletcollisions_skip2
                ;get buffer address of protection bar
                ldy BULLETSYCHARCOL
                lda rotatingLayerLow,Y
                sta BUNKERADDRESSL
                lda rotatingLayerHigh,Y
                sta BUNKERADDRESSH
                ;modify chr at buffer and x offset
                lda HITCHARACTER
                and #LOW_NIBBLE_MASK
                tay
                lda purLayerFrameTable,Y
                ldy #$00
                sta (BUNKERADDRESSL),Y
                cmp #CHR_SPACE
                bne .mothershipbulletcollisions_nospace
                ldy BULLETSYCHARCOL
                lda #$01
                sta xOffsetArray,Y
.mothershipbulletcollisions_nospace
                rts
.mothershipbulletcollisions_skip2
                ;check if boss was hit
                lda HITCHARACTER
                and #%11110000  ;00-0F
                cmp #CHRGROUP_BOSS
                bne .mothershipbulletcollisions_skip3
                ;victory at wave_3
                lda #TRUE
                sta bossWasHit
                ;stops in flight bullets from scoring
                jsr BulletsReset
.mothershipbulletcollisions_skip3
                rts

;********************************************************************
; Subroutine:   MothershipScoreCalculation
;********************************************************************
MothershipScoreCalculation
                lda #$00
                sta MOTHERSHIPSCORE
                ;base score value dep. on X position
                ldx CURRENTBOSSXPOS
                sed
                clc
                lda xPosScoreTable,X
                adc MOTHERSHIPSCORE
                sta MOTHERSHIPSCORE
                cld
                ;add 100 points for each complete destruction
                ldx #$00
.mothershipscorecalculation_loop
                sed
                clc
                lda xOffsetPurLayerArray,X
                adc MOTHERSHIPSCORE
                sta MOTHERSHIPSCORE
                cld
                inx
                cpx #SIZE_OF_PURPLE_LAYER
                bne .mothershipscorecalculation_loop
                rts

;********************************************************************
; Subroutine:   BossUpdateFiring
;********************************************************************
BossUpdateFiring
                ldx #$00
.bossupdatefiring_loop
                lda bossFireArray,X
                sta bossFire
                stx bossTemp
                ldy bossFire
                iny
                sty bossFire
                cpy #BOSS_FIRE_DELAY
                beq .bossupdatefiring_fire
                jmp .bossupdatefiring_dontfire
.bossupdatefiring_fire
                ;boss bullets fire
                ldx #$00
.bossupdatefiring_bulletloop
                lda bulletsActive,X
                bne .bossupdatefiring_bulletskip
                ;save the current bullet in the queue
                lda #$01
                sta bulletsActive,X
                clc
                lda CURRENTBOSSXPOS
                adc #$05
                sta bulletsXChar,X
                lda #CHR_ALIENBULLET1
                sta bulletsYOffset,X
                ldy bossTemp
                lda bossBulletYArray,Y
                sta bulletsYChar,X
                ;boss bullet color
                lda #PUR
                sta bulletsColor,X
                ;boss bullet direction
                lda #FALSE
                sta bulletsDir,X
                ;boss bullet speed
                lda #$03
                sta bulletsSpeed,X
                sta bulletsWait,X
                ;trigger the alien firing sound
                jsr TriggerSoundAlienFiring
                ;found a slot, quit the loop
                jmp .bossupdatefiring_found
.bossupdatefiring_bulletskip
                ;loop for each bullet
                inx
                cpx #BULLETS_MAX
                bne .bossupdatefiring_bulletloop
.bossupdatefiring_found
                lda #$00
                sta bossFire
.bossupdatefiring_dontfire
                ldx bossTemp
                lda bossFire
                sta bossFireArray,X
                ;loop for the 4 Y positions
                inx
                cpx #$04
                bne .bossupdatefiring_loop
                rts

;********************************************************************
;mothership data ****************************************************
;********************************************************************
bossState
                !byte BOSS_STATE_ACTIVE

;Jump table for the boss behavior
bossStateJumpTableLow
                !byte <TreatBossStateActive
                !byte <TreatBossStateHit
                !byte <TreatBossStateExp
                !byte <TreatBossStateScore

bossStateJumpTableHigh
                !byte >TreatBossStateActive
                !byte >TreatBossStateHit
                !byte >TreatBossStateExp
                !byte >TreatBossStateScore

initialMothershipXpos
                !byte 23

initialMothershipYpos
                !byte  3

initialBunkerData
                !byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                !byte $00, $00, $00, $71, $51, $00, $00, $00, $00, $00
                !byte $00, $00, $70, $50, $50, $00, $00, $00, $00, $00
                !byte $00, $00, $73, $50, $51, $00, $00, $00, $00, $00
                !byte $00, $71, $50, $50, $50, $00, $00, $00, $00, $00
                !byte $00, $73, $50, $50, $51, $00, $00, $00, $00, $00
                !byte $70, $50, $50, $50, $50, $00, $00, $00, $00, $00
                !byte $71, $50, $50, $50, $51, $00, $00, $00, $00, $00
                !byte $72, $50, $50, $50, $50, $00, $00, $00, $00, $00
                !byte $73, $50, $50, $50, $51, $00, $00, $00, $00, $00
                !byte $73, $50, $50, $50, $50, $00, $00, $00, $00, $00
                !byte $72, $50, $50, $50, $51, $00, $00, $00, $00, $00
                !byte $71, $50, $50, $50, $50, $00, $00, $00, $00, $00
                !byte $70, $50, $50, $50, $51, $00, $00, $00, $00, $00
                !byte $00, $73, $50, $50, $50, $00, $00, $00, $00, $00
                !byte $00, $71, $50, $50, $51, $00, $00, $00, $00, $00
                !byte $00, $00, $73, $50, $50, $00, $00, $00, $00, $00
                !byte $00, $00, $70, $50, $51, $00, $00, $00, $00, $00
                !byte $00, $00, $00, $71, $50, $00, $00, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

mothershipShapeTable
                !byte $00, $00, $00, $00, $60, $2A, $00, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $2A, $00, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $2A, $00, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $6A, $2A, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $8F, $2A, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $90, $6A, $2A, $00, $00
                !byte $00, $00, $00, $00, $00, $ED, $8F, $2A, $00, $00
                !byte $00, $00, $00, $00, $00, $ED, $90, $6A, $2A, $00
                !byte $00, $00, $00, $00, $00, $ED, $66, $6B, $1B, $2A
                !byte $00, $00, $00, $00, $00, $FC, $F4, $6C, $1C, $2A
                !byte $00, $00, $00, $00, $00, $FD, $F5, $6D, $1D, $2A
                !byte $00, $00, $00, $00, $00, $ED, $66, $6E, $2E, $2A
                !byte $00, $00, $00, $00, $00, $ED, $65, $63, $2A, $00
                !byte $00, $00, $00, $00, $00, $ED, $64, $2A, $00, $00
                !byte $00, $00, $00, $00, $00, $65, $63, $2A, $00, $00
                !byte $00, $00, $00, $00, $00, $64, $2A, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $63, $2A, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $2A, $00, $00, $00, $00
                !byte $00, $00, $00, $00, $00, $2A, $00, $00, $00, $00
                !byte $00, $00, $00, $00, $61, $2A, $00, $00, $00, $00

mothershipColorTable
                !byte $00, $00, $00, $00, YEL6, WHT, $00, $00, $00, $00
                !byte $00, $00, $00, YEL6, PUR, WHT, $00, $00, $00, $00
                !byte $00, $00, YEL6, YEL6, PUR, WHT, $00, $00, $00, $00
                !byte $00, $00, YEL6, YEL6, PUR, ORG5, WHT, $00, $00, $00
                !byte $00, YEL6, YEL6, YEL6, PUR, YGR6, WHT, $00, $00, $00
                !byte $00, YEL6, YEL6, YEL6, PUR, YGR6, ORG5, WHT, $00, $00
                !byte YEL6, YEL6, YEL6, YEL6, PUR, YGR6, YGR6, WHT, $00, $00
                !byte YEL6, YEL6, YEL6, YEL6, PUR, YGR6, YGR6, ORG5, WHT, $00
                !byte YEL6, YEL6, YEL6, YEL6, PUR, YGR6, YGR6, ORG5, YGR6, WHT
                !byte YEL6, YEL6, YEL6, YEL6, PUR, PNK, PUR, ORG5, ORG5, WHT
                !byte YEL6, YEL6, YEL6, YEL6, PUR, PNK, PUR, ORG5, ORG5, WHT
                !byte YEL6, YEL6, YEL6, YEL6, PUR, YGR6, YGR6, ORG5, YGR6, WHT
                !byte YEL6, YEL6, YEL6, YEL6, PUR, YGR6, YGR6, ORG5, WHT, $00
                !byte YEL6, YEL6, YEL6, YEL6, PUR, YGR6, YGR6, WHT, $00, $00
                !byte $00, YEL6, YEL6, YEL6, PUR, YGR6, ORG5, WHT, $00, $00
                !byte $00, YEL6, YEL6, YEL6, PUR, YGR6, WHT, $00, $00, $00
                !byte $00, $00, YEL6, YEL6, PUR, ORG5, WHT, $00, $00, $00
                !byte $00, $00, YEL6, YEL6, PUR, WHT, $00, $00, $00, $00
                !byte $00, $00, $00, YEL6, PUR, WHT, $00, $00, $00, $00
                !byte $00, $00, $00, $00, YEL6, WHT, $00, $00, $00, $00

bunkerAddrTableLow
                !byte 0
                !byte 0
                !byte 0
                !byte <mothershipShapeTable
                !byte <mothershipShapeTable+10
                !byte <mothershipShapeTable+20
                !byte <mothershipShapeTable+30
                !byte <mothershipShapeTable+40
                !byte <mothershipShapeTable+50
                !byte <mothershipShapeTable+60
                !byte <mothershipShapeTable+70
                !byte <mothershipShapeTable+80
                !byte <mothershipShapeTable+90
                !byte <mothershipShapeTable+100
                !byte <mothershipShapeTable+110
                !byte <mothershipShapeTable+120
                !byte <mothershipShapeTable+130
                !byte <mothershipShapeTable+140
                !byte <mothershipShapeTable+150
                !byte <mothershipShapeTable+160
                !byte <mothershipShapeTable+170
                !byte <mothershipShapeTable+180
                !byte <mothershipShapeTable+190

bunkerAddrTableHigh
                !byte 0
                !byte 0
                !byte 0
                !byte >mothershipShapeTable
                !byte >mothershipShapeTable+10
                !byte >mothershipShapeTable+20
                !byte >mothershipShapeTable+30
                !byte >mothershipShapeTable+40
                !byte >mothershipShapeTable+50
                !byte >mothershipShapeTable+60
                !byte >mothershipShapeTable+70
                !byte >mothershipShapeTable+80
                !byte >mothershipShapeTable+90
                !byte >mothershipShapeTable+100
                !byte >mothershipShapeTable+110
                !byte >mothershipShapeTable+120
                !byte >mothershipShapeTable+130
                !byte >mothershipShapeTable+140
                !byte >mothershipShapeTable+150
                !byte >mothershipShapeTable+160
                !byte >mothershipShapeTable+170
                !byte >mothershipShapeTable+180
                !byte >mothershipShapeTable+190

rotatingLayerLow
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte <mothershipShapeTable+14
                !byte <mothershipShapeTable+24
                !byte <mothershipShapeTable+34
                !byte <mothershipShapeTable+44
                !byte <mothershipShapeTable+54
                !byte <mothershipShapeTable+64
                !byte <mothershipShapeTable+74
                !byte <mothershipShapeTable+84
                !byte <mothershipShapeTable+94
                !byte <mothershipShapeTable+104
                !byte <mothershipShapeTable+114
                !byte <mothershipShapeTable+124
                !byte <mothershipShapeTable+134
                !byte <mothershipShapeTable+144
                !byte <mothershipShapeTable+154
                !byte <mothershipShapeTable+164
                !byte <mothershipShapeTable+174
                !byte <mothershipShapeTable+184

rotatingLayerHigh
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte >mothershipShapeTable+14
                !byte >mothershipShapeTable+24
                !byte >mothershipShapeTable+34
                !byte >mothershipShapeTable+44
                !byte >mothershipShapeTable+54
                !byte >mothershipShapeTable+64
                !byte >mothershipShapeTable+74
                !byte >mothershipShapeTable+84
                !byte >mothershipShapeTable+94
                !byte >mothershipShapeTable+104
                !byte >mothershipShapeTable+114
                !byte >mothershipShapeTable+124
                !byte >mothershipShapeTable+134
                !byte >mothershipShapeTable+144
                !byte >mothershipShapeTable+154
                !byte >mothershipShapeTable+164
                !byte >mothershipShapeTable+174
                !byte >mothershipShapeTable+184

rotatingLayerTemp
                !byte 0

alienPilotFrame1
                !byte $F8,$F0,$6C,$1C
                !byte $F9,$F1,$6D,$1D

alienPilotFrame2
                !byte $FA,$F2,$6C,$1C
                !byte $FB,$F3,$6D,$1D

alienPilotFrame3
                !byte $FC,$F4,$6C,$1C
                !byte $FD,$F5,$6D,$1D

alienPilotFrame4
                !byte $FE,$F6,$6C,$1C
                !byte $FF,$F7,$6D,$1D

alienPilotFrame5
                !byte $FE,$F6,$6F,$1C
                !byte $FF,$F7,$67,$1D

alienPilotFrame6
                !byte $FC,$F4,$68,$1C
                !byte $FD,$F5,$69,$1D

alienPilotFrame7
                !byte $FA,$F2,$6C,$5F
                !byte $FB,$F3,$6D,$EE

alienPilotFrame8
                !byte $F8,$F0,$6C,$EF
                !byte $F9,$F1,$6D,$62

alienPilotFrameTableLow
                !byte <alienPilotFrame1
                !byte <alienPilotFrame2
                !byte <alienPilotFrame3
                !byte <alienPilotFrame4
                !byte <alienPilotFrame5
                !byte <alienPilotFrame6
                !byte <alienPilotFrame7
                !byte <alienPilotFrame8

alienPilotFrameTableHigh
                !byte >alienPilotFrame1
                !byte >alienPilotFrame2
                !byte >alienPilotFrame3
                !byte >alienPilotFrame4
                !byte >alienPilotFrame5
                !byte >alienPilotFrame6
                !byte >alienPilotFrame7
                !byte >alienPilotFrame8

initialBunkersEdgeXArray
                !byte 26, 25, 25, 24, 24, 23, 23, 23
                !byte 23, 23, 23, 23, 23, 24, 24, 25
                !byte 25, 26
                !byte 27, 27            ;left and right bumper

currentBunkersEdgeXArray
                !byte 0, 0, 0, 0, 0, 0, 0, 0
                !byte 0, 0, 0, 0, 0, 0, 0, 0
                !byte 0, 0
                !byte 0, 0

currentBunkersEdgeYArray
                !byte  4,  5,  6,  7,  8,  9, 10, 11
                !byte 12, 13, 14, 15, 16, 17, 18, 19
                !byte 20, 21
                !byte  3, 22            ;left and right bumper

bossWasHit
                !byte 0

;4 characters and 1 control byte for edge correction
bunkerDamageFrame1
                !byte $73, $50, $50, $50, $00

bunkerDamageFrame2
                !byte $72, $50, $50, $50, $00

bunkerDamageFrame3
                !byte $71, $50, $50, $50, $00

bunkerDamageFrame4
                !byte $70, $50, $50, $50, $00

bunkerDamageFrame5
                !byte $2A, $73, $50, $50, $01

bunkerDamageFrame6
                !byte $2A, $72, $50, $50, $00

bunkerDamageFrame7
                !byte $2A, $71, $50, $50, $00

bunkerDamageFrame8
                !byte $2A, $70, $50, $50, $00

bunkerDamageFrame9
                !byte $2A, $2A, $73, $50, $01

bunkerDamageFrame10
                !byte $2A, $2A, $72, $50, $00

bunkerDamageFrame11
                !byte $2A, $2A, $71, $50, $00

bunkerDamageFrame12
                !byte $2A, $2A, $70, $50, $00

bunkerDamageFrame13
                !byte $2A, $2A, $2A, $73, $01

bunkerDamageFrame14
                !byte $2A, $2A, $2A, $72, $00

bunkerDamageFrame15
                !byte $2A, $2A, $2A, $71, $00

bunkerDamageFrame16
                !byte $2A, $2A, $2A, $70, $00

bunkerDamageFrame17
                !byte $2A, $2A, $2A, $2A, $01

bunkerDamageFrameTableLow
                !byte <bunkerDamageFrame1
                !byte <bunkerDamageFrame2
                !byte <bunkerDamageFrame3
                !byte <bunkerDamageFrame4
                !byte <bunkerDamageFrame5
                !byte <bunkerDamageFrame6
                !byte <bunkerDamageFrame7
                !byte <bunkerDamageFrame8
                !byte <bunkerDamageFrame9
                !byte <bunkerDamageFrame10
                !byte <bunkerDamageFrame11
                !byte <bunkerDamageFrame12
                !byte <bunkerDamageFrame13
                !byte <bunkerDamageFrame14
                !byte <bunkerDamageFrame15
                !byte <bunkerDamageFrame16
                !byte <bunkerDamageFrame17

bunkerDamageFrameTableHigh
                !byte >bunkerDamageFrame1
                !byte >bunkerDamageFrame2
                !byte >bunkerDamageFrame3
                !byte >bunkerDamageFrame4
                !byte >bunkerDamageFrame5
                !byte >bunkerDamageFrame6
                !byte >bunkerDamageFrame7
                !byte >bunkerDamageFrame8
                !byte >bunkerDamageFrame9
                !byte >bunkerDamageFrame10
                !byte >bunkerDamageFrame11
                !byte >bunkerDamageFrame12
                !byte >bunkerDamageFrame13
                !byte >bunkerDamageFrame14
                !byte >bunkerDamageFrame15
                !byte >bunkerDamageFrame16
                !byte >bunkerDamageFrame17

initialBunkerFrameIndexTable
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte $0E       ;$00, $00, $00, $71
                !byte $0B       ;$00, $00, $70, $50
                !byte $08       ;$00, $00, $73, $50
                !byte $06       ;$00, $71, $50, $50
                !byte $04       ;$00, $73, $50, $50
                !byte $03       ;$70, $50, $50, $50
                !byte $02       ;$71, $50, $50, $50
                !byte $01       ;$72, $50, $50, $50
                !byte $00       ;$73, $50, $50, $50
                !byte $00       ;$73, $50, $50, $50
                !byte $01       ;$72, $50, $50, $50
                !byte $02       ;$71, $50, $50, $50
                !byte $03       ;$70, $50, $50, $50
                !byte $04       ;$00, $73, $50, $50
                !byte $06       ;$00, $71, $50, $50
                !byte $08       ;$00, $00, $73, $50
                !byte $0B       ;$00, $00, $70, $50
                !byte $0E       ;$00, $00, $00, $71

bunkerFrameIndexTable
                !fill SCREEN_HIGHT, 0

;Rotating purple layer, character conversion table:
;50->5D 51->5B 52->5C 53->5D 54->5E 55->57 56->57 57->2A
;58->5B 59->5C 5A->5E 5B->55 5C->2A 5D->56 5E->57
purLayerFrameTable
                !byte $5D, $5B, $5C, $5D, $5E, $57, $57, CHR_SPACE
                !byte $5B, $5C, $5E, $55, CHR_SPACE, $56, $57

xOffsetArray
                !byte 0
                !byte 0
                !byte 0
                !byte 0

xOffsetPurLayerArray
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0

xOffsetTemp
                !byte 0

;base value for the calculation of MOTHERSHIPSCORE (400-7400)
xPosScoreTable
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte 0
                !byte $74
                !byte $74
                !byte $64
                !byte $64
                !byte $54
                !byte $54
                !byte $44
                !byte $44
                !byte $34
                !byte $34
                !byte $24
                !byte $24
                !byte $14
                !byte $14
                !byte $04
                !byte $04

;for the 4 Y positions
bossBulletYArray
                !byte $0B, $0C, $0D, $0E
bossFireArray
                !byte $00, $3C, $78, $B4

bossFire
                !byte 0

bossTemp
                !byte 0

;********************************************************************
; MOTHERSHIP MODULE END                                             *
;********************************************************************