;********************************************************************
; PARTICLE EXPLOSION MODULE                                         *
;********************************************************************
;********************************************************************
; Subroutine:   ParticleExplosionUpdate
;               Used for the player ship explosion and for the
;               mothership one.
;********************************************************************
ParticleExplosionUpdate
                lda particleExplosionTriggered
                cmp #TRUE
                bne .particleexplosionupdate_exit
                ;player explosion handle speed
                ldy #TIMER_A
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .particleexplosionupdate_exit
                jsr RestartTimer
                ;do task ********************************************
                jsr DrawParticleExplosionFrame
                jsr ParticleExplosionUpdateFrame
.particleexplosionupdate_exit
                rts

;********************************************************************
; Subroutine:   DrawParticleExplosionFrame
;********************************************************************
DrawParticleExplosionFrame
.drawparticleexplosionframe_loop
                jsr GetParticleExplosionFields
                lda particleExplosionField01
                cmp #$00
                beq .drawparticleexplosionframe_nextframe
                ;plot one character if not out of screen
                lda particleExplosionField01
                sta CURRENTCHARACTER
                lda particleExplosionField02
                sta CURRENTCOLOR
                clc
                lda particleExplosionX
                adc particleExplosionField03
                sta CURRENTXPOS
                clc
                lda particleExplosionY
                adc particleExplosionField04
                sta CURRENTYPOS
                jsr Plot
                ;inc pointer
                clc
                lda PARTICLEEXPLOSIONPOINTERL
                adc #$04                ;number of fields
                sta PARTICLEEXPLOSIONPOINTERL
                lda PARTICLEEXPLOSIONPOINTERH
                adc #$00
                sta PARTICLEEXPLOSIONPOINTERH
                jmp .drawparticleexplosionframe_loop
.drawparticleexplosionframe_nextframe
                ;inc pointer
                clc
                lda PARTICLEEXPLOSIONPOINTERL
                adc #$04                ;number of fields
                sta PARTICLEEXPLOSIONPOINTERL
                lda PARTICLEEXPLOSIONPOINTERH
                adc #$00
                sta PARTICLEEXPLOSIONPOINTERH
                rts

;********************************************************************
; Subroutine:   ParticleExplosionUpdateFrame
;********************************************************************
ParticleExplosionUpdateFrame
                ;inc frame index
                inc PARTICLEEXPLOSIONINDEX
                lda PARTICLEEXPLOSIONINDEX
                cmp #NO_OF_PARTICLE_EXPOSION_FRAMES
                bne .particleexplosionupdateframe_exit
                ;animation finished
                lda #$00
                sta PARTICLEEXPLOSIONINDEX
                sta particleExplosionTriggered
                ldy #TIMER_A
                jsr ResetTimer
                ;reset pointer
                lda #<particleExplosionTable
                sta PARTICLEEXPLOSIONPOINTERL
                lda #>particleExplosionTable
                sta PARTICLEEXPLOSIONPOINTERH
.particleexplosionupdateframe_exit
                rts

;********************************************************************
; Subroutine:   GetParticleExplosionFields
;********************************************************************
GetParticleExplosionFields
                ldy #$00
.getparticleexplosionfields_loop
                lda (PARTICLEEXPLOSIONPOINTERL),Y
                sta particleExplosionField01,Y
                iny
                cpy #$04                    ;for all explosion fields
                bne .getparticleexplosionfields_loop
                rts

;********************************************************************
;Particle explosion data ********************************************
;********************************************************************
particleExplosionTriggered
                !byte FALSE

particleExplosionX
                !byte 0

particleExplosionY
                !byte 0

particleExplosionField01
                !byte 0         ;Character code or end of data

particleExplosionField02
                !byte 0         ;Color code

particleExplosionField03
                !byte 0         ;X offset

particleExplosionField04
                !byte 0         ;Y offset

;particleExplosionField01, particleExplosionField02, particleExplosionField03, particleExplosionField04
particleExplosionTable
                !byte 058, WHT,    -1, -1   ;A
                !byte 056, WHT,     1,  0
                !byte 042, WHT,     0,  0
                !byte 042, WHT,     0,  1
                !byte 042, WHT,     1,  1
                !byte 068, ORG,     2,  1
                !byte 059, WHT,    -1,  2
                !byte 057, WHT,     1,  2
                !byte 000, 000,     0,  0
                !byte 116, RED5,    0, -1   ;B
                !byte 117, YEL6,    1, -1
                !byte 118, WHT,    -1,  0
                !byte 119, RED5,    0,  0
                !byte 120, PNK,     2,  0
                !byte 124, RED5,    2,  1
                !byte 121, ORG,    -1,  1
                !byte 122, YEL6,    0,  1
                !byte 123, LGRN,    1,  1
                !byte 125, GRN,     0,  2
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 058, WHT,     0, -2   ;C
                !byte 126, RED5,    0, -1
                !byte 127, YEL6,    1, -1
                !byte 042, WHT,    -1, -1
                !byte 056, WHT,     3, -1
                !byte 128, LGRN,   -1,  0
                !byte 129, YEL6,    0,  0
                !byte 130, RED5,   -1,  1
                !byte 131, ORG,     0,  1
                !byte 078, ORG,     1,  0
                !byte 070, PNK,     2,  1
                !byte 079, RED5,    4,  1
                !byte 042, WHT,    -1,  2
                !byte 080, YEL6,    1,  2
                !byte 000, 000,     0,  0
                !byte 081, YGRN,   -1, -1   ;D
                !byte 082, RED5,    2, -1
                !byte 071, YEL6,    2,  1
                !byte 083, PNK,    -1,  2
                !byte 084, LGRN,    2,  2
                !byte 132, RED5,   -1,  0
                !byte 123, PNK,     0,  0
                !byte 123, YGRN,    1,  0
                !byte 133, PNK,     1,  1
                !byte 134, GRN,    -1,  1
                !byte 116, ORG,     0,  1
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 058, WHT,     0, -3   ;E
                !byte 042, WHT,     0, -2
                !byte 085, GRN,    -2, -1
                !byte 042, WHT,     3, -1
                !byte 056, WHT,     5, -1
                !byte 072, GRN,     2,  1
                !byte 042, WHT,     4,  1
                !byte 086, ORG,     6,  1
                !byte 087, RED5,   -2,  2
                !byte 059, WHT,     0,  4
                !byte 057, WHT,     5,  4
                !byte 117, GRN,    -1,  0
                !byte 118, WHT,     0,  0
                !byte 123, YGRN,    1,  0
                !byte 133, YEL6,    1,  1
                !byte 120, YGRN,   -1,  1
                !byte 121, ORG,     0,  1
                !byte 000, 000,     0,  0
                !byte 042, WHT,    -1, -1   ;F
                !byte 042, WHT,     2, -1
                !byte 073, YGRN,    2,  1
                !byte 042, WHT,    -1,  2
                !byte 042, WHT,     2,  2
                !byte 122, ORG,    -1,  0
                !byte 123, YGRN,    0,  0
                !byte 124, WHT,     1,  0
                !byte 134, YEL6,    1,  1
                !byte 124, ORG,    -1,  1
                !byte 125, GRN,     0,  1
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 088, YEL6,    0, -4   ;G
                !byte 042, WHT,     0, -3
                !byte 089, YGRN,   -2, -2
                !byte 090, RED5,    2, -2
                !byte 056, WHT,     7, -2
                !byte 042, WHT,    -2, -1
                !byte 042, WHT,     5, -1
                !byte 091, YEL6,    3,  0
                !byte 089, YGRN,   -1,  0
                !byte 090, RED5,    0,  0
                !byte 091, YEL6,   -1,  1
                !byte 092, ORG,     0,  1
                !byte 074, WHT,     2,  1
                !byte 042, WHT,     6,  1
                !byte 092, YGRN,    8,  1
                !byte 042, WHT,    -2,  2
                !byte 093, RED5,    2,  2
                !byte 094, PNK,    -2,  3
                !byte 095, YEL6,    1,  3
                !byte 042, WHT,     0,  4
                !byte 042, WHT,     5,  4
                !byte 057, WHT,     7,  4
                !byte 059, WHT,     0,  5
                !byte 000, 000,     0,  0
                !byte 042, WHT,     0, -1   ;H
                !byte 042, WHT,     1, -1
                !byte 042, WHT,    -1,  0
                !byte 042, WHT,     2,  0
                !byte 042, WHT,    -1,  1
                !byte 042, WHT,     2,  1
                !byte 042, WHT,     0,  2
                !byte 042, WHT,     1,  2
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 058, WHT,     0, -5   ;I
                !byte 042, WHT,     0, -4
                !byte 096, YGRN,   -2, -3
                !byte 097, RED5,    2, -3
                !byte 042, WHT,    -2, -2
                !byte 042, WHT,     2, -2
                !byte 042, WHT,     7, -2
                !byte 056, WHT,     9, -2
                !byte 098, PNK,     4, -1
                !byte 042, WHT,     3,  0
                !byte 042, WHT,     8,  1
                !byte 099, YEL6,    9,  1
                !byte 042, WHT,     2,  2
                !byte 100, YGRN,    3,  2
                !byte 042, WHT,    -2,  3
                !byte 042, WHT,     1,  3
                !byte 101, RED5,   -3,  4
                !byte 102, YEL6,    2,  4
                !byte 042, WHT,     7,  4
                !byte 042, WHT,     0,  5
                !byte 057, WHT,     9,  5
                !byte 059, WHT,    -1,  6
                !byte 000, 000,     0,  0
                !byte 042, WHT,     0,  0   ;J
                !byte 042, WHT,     1,  0
                !byte 042, WHT,     0,  1
                !byte 042, WHT,     1,  1
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 103, YGRN,   -1, -6   ;K
                !byte 042, WHT,     0, -5
                !byte 104, RED5,   -2, -4
                !byte 105, YEL6,    3, -4
                !byte 042, WHT,    -2, -3
                !byte 042, WHT,     2, -3
                !byte 056, WHT,    11, -3
                !byte 106, YGRN,    5, -2
                !byte 042, WHT,     9, -2
                !byte 042, WHT,     4, -1
                !byte 042, WHT,     9,  1
                !byte 107, RED5,   10,  1
                !byte 042, WHT,     3,  2
                !byte 108, YEL6,    4,  2
                !byte 042, WHT,    -3,  4
                !byte 042, WHT,     2,  4
                !byte 109, YGRN,   -3,  5
                !byte 110, RED5,    3,  5
                !byte 042, WHT,     9,  5
                !byte 042, WHT,    -1,  6
                !byte 057, WHT,    10,  6
                !byte 059, WHT,    -1,  7
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 058, WHT,    -2, -7   ;L
                !byte 042, WHT,    -1, -6
                !byte 111, YEL6,   -3, -5
                !byte 112, YGRN,    3, -5
                !byte 056, WHT,    12, -5
                !byte 042, WHT,    -2, -4
                !byte 042, WHT,     3, -4
                !byte 113, RED5,    6, -3
                !byte 042, WHT,    11, -3
                !byte 042, WHT,     5, -2
                !byte 042, WHT,    10,  1
                !byte 114, YEL6,   11,  1
                !byte 042, WHT,     4,  2
                !byte 115, YGRN,    5,  3
                !byte 042, WHT,    -3,  5
                !byte 042, WHT,     3,  5
                !byte 068, RED5,   -3,  6
                !byte 069, YEL6,    4,  6
                !byte 042, WHT,    10,  6
                !byte 042, WHT,    -1,  7
                !byte 057, WHT,    11,  7
                !byte 059, WHT,    -2,  8
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 042, WHT,    -2, -7   ;M
                !byte 058, WHT,    -3, -7
                !byte 056, WHT,    11, -7
                !byte 070, YGRN,    2, -6
                !byte 042, WHT,    -3, -5
                !byte 042, WHT,     3, -5
                !byte 042, WHT,    12, -5
                !byte 071, RED5,    6, -4
                !byte 042, WHT,     6, -3
                !byte 042, WHT,    11,  1
                !byte 072, YEL6,   12,  1
                !byte 042, WHT,     5,  3
                !byte 073, YEL6,    6,  4
                !byte 042, WHT,    -3,  6
                !byte 042, WHT,     4,  6
                !byte 042, WHT,    11,  7
                !byte 074, YGRN,    3,  7
                !byte 042, WHT,    -2,  8
                !byte 057, WHT,    11,  8
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0
                !byte 042, WHT,    -3, -7   ;N
                !byte 042, WHT,    11, -7
                !byte 042, WHT,     2, -6
                !byte 042, WHT,     6, -4
                !byte 042, WHT,    12,  1
                !byte 042, WHT,     6,  4
                !byte 042, WHT,     3,  7
                !byte 042, WHT,    11,  8
                !byte 000, 000,     0,  0
                !byte 000, 000,     0,  0

;********************************************************************
; PARTICLE EXPLOSION MODULE END                                     *
;********************************************************************