;********************************************************************
; PLAYER MODULE                                                     *
;********************************************************************
;********************************************************************
; Subroutine:   PlayerInit
;********************************************************************
PlayerInit
                lda #PLAYER_START_X
                sta playerX
                lda #PLAYER_START_Y
                sta playerY
                lda #FALSE
                sta PLAYERACTIVE
                rts

;********************************************************************
; Subroutine:   PlayerReset
;********************************************************************
PlayerReset
                lda #TRUE
                sta PLAYERACTIVE
                jsr SetShip
                rts

;********************************************************************
; Subroutine:   PlayerUpdate
;********************************************************************
PlayerUpdate
                lda PLAYERACTIVE
                beq .playerupdate_exit
                ;player update frame
                ldy #TIMER_2
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .playerupdate_noupdate
                jsr RestartTimer
                ;do task ********************************************
                ;rotate player frame
                lda playerFrameChangeAllowed
                cmp #TRUE
                bne .playerupdate_noupdate
                inc PLAYERFRAME
                lda PLAYERFRAME
                cmp #$04
                bne .playerupdate_noupdate
                lda #$00
                sta PLAYERFRAME
.playerupdate_noupdate
                ;player update movement
                ldy #TIMER_1
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .playerupdate_nomovement
                jsr RestartTimer
                ;do task ********************************************
                jsr PlayerUpdatePosition
                jsr SetShip
                ;align player positions with bullets position
                ldy playerX
                iny
                iny
                sty PLAYERXCHAR
                ldy playerY
                iny
                sty PLAYERYCHAR
                lda #$00
                sta PLAYERXOFFSET
                sta PLAYERYOFFSET
.playerupdate_nomovement
                ;player update firing
                ;do fire after the ship has been clamped to position
                ;so that the bullet lines up
                jsr GetFirePressed
                bne .playerupdate_nofire
                ;fire the bullet
                jsr PlayerBulletsFire
                ;trigger the firing sound
                jsr TriggerSoundPlayerFiring
.playerupdate_nofire
                jsr PlayerBulletCollisions
                jsr PlayerAlienCollisions
.playerupdate_exit
                rts

;********************************************************************
; Subroutine:   PlayerBulletCollisions
;********************************************************************
PlayerBulletCollisions
                ;check left part of ship
                lda PLAYERXCHAR
                sta BULLETSXCHARCOL
                lda PLAYERYCHAR
                sta BULLETSYCHARCOL
                dec BULLETSYCHARCOL
                lda #FALSE
                sta BULLETSDIRCOL
                jsr BulletsCollided
                bne .playerupdatecollisions_collision
                ;check right part of ship
                lda PLAYERXCHAR
                sta BULLETSXCHARCOL
                lda PLAYERYCHAR
                sta BULLETSYCHARCOL
                lda #FALSE
                sta BULLETSDIRCOL
                jsr BulletsCollided
                beq .playerupdatecollisions_nocollision
.playerupdatecollisions_collision
                ;check for invulnerability
                lda PLAYERINVULNERABLE
                cmp #TRUE
                bne .playerupdatecollisions_noprotection
                ;the shield it protecting
                rts
.playerupdatecollisions_noprotection
                lda #FALSE
                sta PLAYERACTIVE
                lda playerX
                sta particleExplosionX
                lda playerY
                sta particleExplosionY
                ;trigger explosion animation
                lda #TRUE
                sta particleExplosionTriggered
                ;start animation speed timer
                ldy #TIMER_A
                jsr StartTimer
                ;trigger particle explosion sound
                jsr TriggerSoundParticleExplosion
                ;stops in flight bullets from scoring
                jsr BulletsReset
                dec PLAYER1LIVES
                bne .playerupdatecollisions_haslivesleft
                jsr UpdateHiScore
.playerupdatecollisions_haslivesleft
                jsr LivesDisplay
                ;change state to: DYING
                lda #FLOW_STATE_DYING
                sta FLOWSTATE
.playerupdatecollisions_nocollision
                rts

;********************************************************************
; Subroutine:   PlayerAlienCollisions
;********************************************************************
PlayerAlienCollisions
                lda PLAYERALIENCOLLISION
                cmp #TRUE
                bne .playeraliencollisions_exit
                ;alien has hit player ship
                lda #FALSE
                sta PLAYERALIENCOLLISION
                sta PLAYERACTIVE
                lda playerX
                sta particleExplosionX
                lda playerY
                sta particleExplosionY
                ;trigger explosion animation
                lda #TRUE
                sta particleExplosionTriggered
                ;start animation speed timer
                ldy #TIMER_A
                jsr StartTimer
                ;trigger particle explosion sound
                jsr TriggerSoundParticleExplosion
                ;stops in flight bullets from scoring
                jsr BulletsReset
                dec PLAYER1LIVES
                bne .playeraliencollisions_haslivesleft
                jsr UpdateHiScore
.playeraliencollisions_haslivesleft
                jsr LivesDisplay
                ;change state to: DYING
                lda #FLOW_STATE_DYING
                sta FLOWSTATE
.playeraliencollisions_exit
                rts

;********************************************************************
; Subroutine:   PlayerUpdatePosition
;********************************************************************
PlayerUpdatePosition
                lda playerMoveAllowed
                cmp #TRUE
                bne .playerupdateposition_else
                ;save current ship position (for DelShip)
                lda playerY
                sta PLAYERYOLD
                lda GAMEPORTTHISFRAME
                and #LEFT_MASK
                bne .playerupdateposition_notleft
                ;left ***********************************************
                lda playerY
                cmp #PLAYER_Y_MIN
                beq .playerupdateposition_else
                ;del the old ship
                jsr DelShip
                ;y position -1
                dec playerY
.playerupdateposition_notleft
                lda GAMEPORTTHISFRAME
                and #RIGHT_MASK
                bne .playerupdateposition_else
                ;right **********************************************
                lda playerY
                cmp #PLAYER_Y_MAX
                beq .playerupdateposition_else
                ;del the old ship
                jsr DelShip
                ;y position + 1
                inc playerY
.playerupdateposition_else
                rts

;********************************************************************
; Subroutine:   SetShip
; Input:        playerX
;               playerY
;               PLAYERFRAME
;********************************************************************
SetShip
                lda playerX
                sta CURRENTSHAPEXPOS
                lda playerY
                sta CURRENTSHAPEYPOS
                ldx PLAYERFRAME
                lda shipFrameLow,X
                sta STRINGL
                lda shipFrameHigh,X
                sta STRINGH
                lda shipColorFrameLow,X
                sta TEXTCOLORL
                lda shipColorFrameHigh,X
                sta TEXTCOLORH
                lda #$02
                sta BLOCKWIDTH
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH
                sta MAXSCREENX
                jsr DrawShape
                rts

;********************************************************************
; Subroutine:   DelShip
; Input:        playerX
;               PLAYERYOLD
;********************************************************************
DelShip
                ldy PLAYERYOLD
                clc
                lda screenRAMRowLow,Y
                adc playerX
                sta SCREENRAML
                lda screenRAMRowHigh,Y
                adc #$00
                sta SCREENRAMH
                ldy #$00
                lda #CHR_SPACE
                sta (SCREENRAML),Y
                iny
                sta (SCREENRAML),Y
                ldy #SCREEN_WIDTH
                sta (SCREENRAML),Y
                iny
                sta (SCREENRAML),Y
                lda SCREENRAMH
                eor #>(SCREENRAM XOR COLORRAM)
                sta SCREENRAMH
                lda #WHT
                ldy #$00
                sta (SCREENRAML),Y
                iny
                sta (SCREENRAML),Y
                ldy #SCREEN_WIDTH
                sta (SCREENRAML),Y
                iny
                sta (SCREENRAML),Y
                rts

;********************************************************************
;player data ********************************************************
;********************************************************************
playerX
                !byte PLAYER_START_X
playerY
                !byte PLAYER_START_Y

playerFrameChangeAllowed
                !byte TRUE    ;FALSE during active shield

playerMoveAllowed
                !byte TRUE    ;FALSE during active shield

; frame 1: big player ship
ship1
                !byte $3A,$38
                !byte $3B,$39

ship1Color
                !byte BGRN,BGRN
                !byte BGR4,BGR4

; frame 2: medium player ship
ship2
                !byte $3E,$3C
                !byte $3F,$3D

ship2Color
                !byte BGR4,BGR4
                !byte BGRN,BGRN

; frame 3: small player ship
ship3
                !byte $42,$40
                !byte $43,$41

ship3Color
                !byte BGRN,BGRN
                !byte BGR4,BGR4

; ship frame table
shipFrameLow
                !byte <ship1
                !byte <ship2
                !byte <ship3
                !byte <ship2

shipFrameHigh
                !byte >ship1
                !byte >ship2
                !byte >ship3
                !byte >ship2

shipColorFrameLow
                !byte <ship1Color
                !byte <ship2Color
                !byte <ship3Color
                !byte <ship2Color

shipColorFrameHigh
                !byte >ship1Color
                !byte >ship2Color
                !byte >ship3Color
                !byte >ship2Color

;********************************************************************
; PLAYER MODULE END                                                 *
;********************************************************************