;********************************************************************
; SCREEN MODULE                                                     *
;********************************************************************
;********************************************************************
; Subroutine:   ClearScreen
;********************************************************************
ClearScreen
                ;fill 1000 bytes (40x25) of screen memory
                lda #CHR_SPACE
                ldx #$FA
.clearscreen_loopscreen
                dex
                sta SCREENRAM,x
                sta SCREENRAM+250,x
                sta SCREENRAM+500,x
                sta SCREENRAM+750,x
                bne .clearscreen_loopscreen
                rts

;********************************************************************
; Subroutine:   SetCharPos
; Input:        CURRENTYPOS    Y Position 0-24 (Address)
; Output:       Y       CURRENTXPOS
; User regs:    A, Y
;********************************************************************
SetCharPos
                ldy CURRENTYPOS        ;load y position as index into list
                lda screenRAMRowLow,Y  ;load low address byte
                sta SCREENL
                lda screenRAMRowHigh,Y ;load high address byte
                sta SCREENH
                ldy CURRENTXPOS        ;load x position into Y register
                rts

;********************************************************************
; Subroutine:   SetColorPos
; Input:        CURRENTYPOS    Y Position 0-24 (Address)
; Output:       Y       CURRENTXPOS
; User regs:    A, Y
;********************************************************************
SetColorPos
                ldy CURRENTYPOS        ;load y position as index into list
                lda colorRAMRowLow,Y   ;load low address byte
                sta SCREENL
                lda colorRAMRowHigh,Y  ;load high address byte
                sta SCREENH
                ldy CURRENTXPOS        ;load x position into Y register
                rts

;********************************************************************
; Subroutine:   IncColumn
; Input:        column
;               SCREENL
;               SCREENH
; Output:       SCREENL
;               SCREENH
; User regs:    A
;********************************************************************
IncColumn
                ;add 40 to current screen Y pos.
                clc
                lda SCREENL
                adc column
                sta SCREENL
                lda SCREENH
                adc column+1
                sta SCREENH
                rts

;********************************************************************
; Subroutine:   DrawTextRotated
; Input:        CURRENTXPOS     X Position 0-39
;               CURRENTYPOS     Y Position 0-24
;               STRINGL         0 terminated string (Address) low
;               STRINGH         0 terminated string (Address) high
;               TEXTCOLORL      0 terminated Text Color (Address) low
;               TEXTCOLORH      0 terminated Text Color (Address) high
; User regs:    A, X, Y
;********************************************************************
DrawTextRotated
                ;text
                ;set cursor
                jsr SetCharPos
                ;get character from string
                ldy #$00
.drawtextrotated_loop
                lda (STRINGL),Y
                sta CURRENTCHARACTER
                cmp #$00
                beq .drawtextrotated_done
                ;*** PRINT CHARACTER ON SCREEN ***
                ;push regs
                tya
                pha
                ldy CURRENTXPOS             ;get column (X pos)
                lda CURRENTCHARACTER        ;get current character
                sta (SCREENL),Y             ;write character to screen ram
                ;pop regs
                pla
                tay
                iny
                jsr IncColumn               ;add 40
                jmp .drawtextrotated_loop
                ;color
                ;set cursor
.drawtextrotated_done
                jsr SetColorPos
                ;get color from string
                ldy #$00
.drawtextrotated_loop2
                lda (TEXTCOLORL),Y
                sta CURRENTCOLOR
                cmp #$00
                beq .drawtextrotated_done2
                ;*** PRINT COLOR ON SCREEN ***
                ;push regs
                tya
                pha
                ldy CURRENTXPOS             ;get column (X pos)
                lda CURRENTCOLOR            ;get current color
                sta (SCREENL),Y             ;write color to color ram
                ;pop regs
                pla
                tay
                iny
                jsr IncColumn               ;add 40
                jmp .drawtextrotated_loop2
.drawtextrotated_done2
                rts

;********************************************************************
; Subroutine:   DrawDecimalRotated
;********************************************************************
DrawDecimalRotated
                ;set cursor for decimal value
                jsr SetCharPos
                ;get high nibble
                lda CURRENTDECNUMBER
                and #HIGH_NIBBLE_MASK
                ;convert to ascii
                lsr
                lsr
                lsr
                lsr
                ora #CHR_ZERO
                sta (SCREENL),Y
                ;move along to next screen position
                jsr IncColumn
                ;get low nibble
                lda CURRENTDECNUMBER
                and #LOW_NIBBLE_MASK
                ;convert to ascii
                ora #CHR_ZERO 
                sta (SCREENL),Y
                ;now set the colors
                jsr SetColorPos
                lda CURRENTCOLOR
                sta (SCREENL),Y
                ;move along to next screen position
                jsr IncColumn
                lda CURRENTCOLOR
                sta (SCREENL),Y
.drawdecimalrotated_skip
                rts

;********************************************************************
; Subroutine:   DrawAnimation
;               Uses hard 'wait' subroutine and does not take care
;               about interrupt or other concurrency!
;********************************************************************
DrawAnimation
                pha
                txa
                pha
                tya
                pha
                lda #CHR_ASTERISK
                sta SPIRALFILLCHAR
                ldx SPIRALFILLCOLORARRAYINDEX
                lda spiralFillColorArray,X
                sta SPIRALFILLCOLOR
                jsr DrawSpiralFill
                inc SPIRALFILLCOLORARRAYINDEX
                lda SPIRALFILLCOLORARRAYINDEX
                cmp #$04
                bne .drawanimation_noreset
                lda #$00
                sta SPIRALFILLCOLORARRAYINDEX
.drawanimation_noreset
                lda #CHR_SPACE
                sta SPIRALFILLCHAR
                lda #WHT
                sta SPIRALFILLCOLOR
                jsr DrawSpiralFill
                pla
                tay
                pla
                tax
                pla
                rts

;********************************************************************
;spiral fill animation data *****************************************
;********************************************************************
spiralFillColorArray
                !byte CYN6,YEL6,PUR,WHT

;********************************************************************
; Subroutine:   DrawSpiralFill
;********************************************************************
DrawSpiralFill
                lda #<SCREENRAM+12*41+6
                sta SPIRALFILLSCREENADDRL
                lda #<COLORRAM+12*41+6
                sta SPIRALFILLCOLORADDRL
                lda #>SCREENRAM+12*41+6
                sta SPIRALFILLSCREENADDRH
                lda #>COLORRAM+12*41+6
                sta SPIRALFILLCOLORADDRH
                lda #$01
                sta SPIRALFILLWIDTH
                lda #$01
                sta SPIRALFILLHEIGHT
.drawspiralfill_loop
                lda SPIRALFILLSCREENADDRL
                sta SPIRALSCREENL
                lda SPIRALFILLCOLORADDRL
                sta SPIRALCOLORL
                lda SPIRALFILLSCREENADDRH
                sta SPIRALSCREENH
                lda SPIRALFILLCOLORADDRH
                sta SPIRALCOLORH
                lda SPIRALFILLWIDTH
                sta SPIRALWIDTH
                lda SPIRALFILLHEIGHT
                sta SPIRALHEIGHT
                jsr DrawRectangleFill
                jsr Wait500
                inc SPIRALFILLWIDTH
                inc SPIRALFILLWIDTH
                inc SPIRALFILLHEIGHT
                inc SPIRALFILLHEIGHT
                lda SPIRALFILLSCREENADDRL
                sbc #SCREEN_WIDTH+1
                sta SPIRALFILLSCREENADDRL
                lda SPIRALFILLSCREENADDRH
                sbc #$00
                sta SPIRALFILLSCREENADDRH
                lda SPIRALFILLCOLORADDRL
                sbc #SCREEN_WIDTH+1
                sta SPIRALFILLCOLORADDRL
                lda SPIRALFILLCOLORADDRH
                sbc #$00
                sta SPIRALFILLCOLORADDRH
                lda SPIRALFILLHEIGHT
                cmp #SCREEN_HIGHT+2
                bne .drawspiralfill_loop
                jsr Wait500
                rts

;********************************************************************
; Subroutine:   DrawRectangleFill
;********************************************************************
DrawRectangleFill
.drawrectanglefill_loop
                ldy #$00
                ;fill a line
.drawrectanglefill_loop2
                lda SPIRALFILLCHAR
                sta (SPIRALSCREENL),Y
                lda SPIRALFILLCOLOR
                sta (SPIRALCOLORL),Y
                iny
                cpy SPIRALWIDTH
                bne .drawrectanglefill_loop2
                dec SPIRALHEIGHT
                ;finished?
                beq .drawrectanglefill_exit
                lda SPIRALSCREENL
                clc
                ;next line screen
                adc #SCREEN_WIDTH
                sta SPIRALSCREENL
                lda SPIRALSCREENH
                adc #$00
                sta SPIRALSCREENH
                clc
                lda SPIRALCOLORL
                clc
                ;next line color
                adc #SCREEN_WIDTH
                sta SPIRALCOLORL
                lda SPIRALCOLORH
                adc #$00
                sta SPIRALCOLORH
                clc
                bcc .drawrectanglefill_loop
.drawrectanglefill_exit
                rts

;********************************************************************
; Subroutine:   DrawShape
; Input:        STRINGL             pointer to string table base
;               TEXTCOLORL          pointer to color table base
;               CURRENTSHAPEXPOS:   X position (0..39)
;               CURRENTSHAPEYPOS:   Y position (0..24)
;               BLOCKWIDTH
;               BLOCKHEIGHT
;               MAXSCREENX
;********************************************************************
DrawShape
                tya
                pha
                lda #$00
                sta SHAPEDATAINDEX
                lda CURRENTSHAPEYPOS
                sta CHRYPOS
.drawshape_nextline
                lda CURRENTSHAPEXPOS
                sta CHRXPOS
.drawshape_posallowed
                lda CHRXPOS
                ;X position >= 0 ?
                bmi .drawshape_nextchr
                ;X position < 37 ?
                cmp MAXSCREENX
                bpl .drawshape_nextchr
                ;Y position >= 0 ?
                ldy CHRYPOS
                bmi .drawshape_nextchr
                ;Y position < 24 ?
                cpy #SCREEN_HIGHT
                bpl .drawshape_nextchr
                ;set screen ram
                clc
                adc screenRAMRowLow, y
                sta SCREENRAML
                sta COLORRAML
                lda #$00
                adc screenRAMRowHigh, y
                sta SCREENRAMH
                eor #>(SCREENRAM XOR COLORRAM)
                sta COLORRAMH
                ;write to screen/color ram
                ldy SHAPEDATAINDEX
                lda (STRINGL),Y
                beq .drawshape_nextchr   ;skip if CHR code 0
                ldy #$00
                sta (SCREENRAML),Y
                ldy SHAPEDATAINDEX
                lda (TEXTCOLORL),Y
                ldy #$00
                sta (COLORRAML),Y
                ;move to next character
.drawshape_nextchr
                inc SHAPEDATAINDEX
                inc CHRXPOS
                ;end of line reached?
                clc
                lda CURRENTSHAPEXPOS
                clc
                adc BLOCKWIDTH
                cmp CHRXPOS
                bne .drawshape_posallowed
                inc CHRYPOS
                dec BLOCKHEIGHT
                ;end of block reached ?
                beq .drawshape_exit
                ;next block line
                bne .drawshape_nextline
.drawshape_exit
                pla
                tay
                rts

;********************************************************************
; Subroutine:   DrawShapeCollisionCheck
; Input:        STRINGL             pointer to string table base
;               TEXTCOLORL          pointer to color table base
;               CURRENTSHAPEXPOS:   X position (0..39)
;               CURRENTSHAPEYPOS:   Y position (0..24)
;               BLOCKWIDTH
;               BLOCKHEIGHT
;               MAXSCREENX
;********************************************************************
DrawShapeCollisionCheck
                txa
                pha
                tya
                pha
                lda #$00
                sta SHAPEDATAINDEX
                lda CURRENTSHAPEYPOS
                sta CHRYPOS
.drawshapecollisioncheck_nextline
                lda CURRENTSHAPEXPOS
                sta CHRXPOS
.drawshapecollisioncheck_posallowed
                lda CHRXPOS
                ;X position >= 0 ?
                bmi .drawshapecollisioncheck_nextchr
                ;X position < 37 ?
                cmp MAXSCREENX
                bpl .drawshapecollisioncheck_nextchr
                ;Y position >= 0 ?
                ldy CHRYPOS
                bmi .drawshapecollisioncheck_nextchr
                ;Y position < 24 ?
                cpy #SCREEN_HIGHT
                bpl .drawshapecollisioncheck_nextchr
                ;set screen ram
                clc
                adc screenRAMRowLow, y
                sta SCREENRAML
                sta COLORRAML
                lda #$00
                adc screenRAMRowHigh, y
                sta SCREENRAMH
                eor #>(SCREENRAM XOR COLORRAM)
                sta COLORRAMH
                ;collision check
                jsr CollisionCheck
                ;write to screen/color ram
                ldy SHAPEDATAINDEX
                lda (STRINGL),Y
                beq .drawshapecollisioncheck_nextchr   ;skip if CHR code 0
                ldy #$00
                sta (SCREENRAML),Y
                ldy SHAPEDATAINDEX
                lda (TEXTCOLORL),Y
                ldy #$00
                sta (COLORRAML),Y
                ;move to next character
.drawshapecollisioncheck_nextchr
                inc SHAPEDATAINDEX
                inc CHRXPOS
                ;end of line reached?
                clc
                lda CURRENTSHAPEXPOS
                clc
                adc BLOCKWIDTH
                cmp CHRXPOS
                bne .drawshapecollisioncheck_posallowed
                inc CHRYPOS
                dec BLOCKHEIGHT
                ;end of block reached ?
                beq .drawshapecollisioncheck_exit
                ;next block line
                bne .drawshapecollisioncheck_nextline
.drawshapecollisioncheck_exit
                pla
                tay
                pla
                tax
                rts

;********************************************************************
; Subroutine:   CollisionCheck
;********************************************************************
CollisionCheck
                ldy #$00
                lda (COLORRAML),Y
                cmp #SHIP_COLOR_1       ;BGR4
                beq .collisioncheck_alienplayer
                cmp #SHIP_COLOR_2       ;BGRN
                beq .collisioncheck_alienplayer
                cmp #SHIP_COLOR_3       ;PNK3
                beq .collisioncheck_alienshield
                cmp #SHIP_COLOR_4       ;PNK5
                beq .collisioncheck_alienshield
                lda (SCREENRAML),Y
                cmp #CHR_SHIELD_START
                bcc .collisioncheck_nocoll
                cmp #CHR_SHIELD_END
                bcc .collisioncheck_alienshield
.collisioncheck_nocoll
                rts
                ;****************************************************
                ;Collision check: alien - player ********************
                ;****************************************************
                ;alien has hit player
.collisioncheck_alienplayer
                lda #TRUE
                sta PLAYERALIENCOLLISION
                ;****************************************************
                ;Collision check: alien - player during shield ******
                ;****************************************************
.collisioncheck_alienshield
                ldx CURRENTALIENNUMBER
                lda alienCollisionArray,X
                cmp #FALSE
                bne .collisioncheck_exit
                lda #TRUE
                sta alienCollisionArray,X
                lda CURRENTSHAPEXPOS
                sta alienExplosionXArray,X
                lda CURRENTSHAPEYPOS
                sta alienExplosionYArray,X
.collisioncheck_exit
                rts

;********************************************************************
; Subroutine:   PrintCharacter
;               Only used for the 'slow print' during intro.
;********************************************************************
PrintCharacter
                ;get one character code
                ldy TITLECOLUMNINDEX
                lda (TITLECHRPOINTER),Y
                sta CURRENTCHARACTER
                sty CURRENTYPOS
                ldx TITLECHRLINEINDEX
                lda gameIntroTextLines,X
                sta CURRENTXPOS
                jsr SetCharPos
                ;print the charcacter
                tya
                pha
                ldy CURRENTXPOS
                lda CURRENTCHARACTER
                sta (SCREENL),Y
                pla
                tay
                ;get one color code
                ldy TITLECOLUMNINDEX
                lda (TITLECOLORPOINTER),Y
                sta CURRENTCOLOR
                ;get color ram address
                ldx TITLECHRLINEINDEX
                lda gameIntroTextLines,X
                sta CURRENTXPOS
                jsr SetColorPos
                ;print the color
                tya
                pha
                ldy CURRENTXPOS
                lda CURRENTCOLOR
                sta (SCREENL),Y
                pla
                tay
                rts

;********************************************************************
; Subroutine:   Plot
; Input:        CURRENTCHARACTER
;               CURRENTCOLOR
;               CURRENTXPOS
;               CURRENTYPOS
;********************************************************************
Plot
                ;X position >= 0 ?
                lda CURRENTXPOS
                cmp #$00
                bcc .plot_exit     ;branch if X < 0
                ;X position < 38 ?
                lda CURRENTXPOS
                cmp #SCREEN_WIDTH-2
                bcs .plot_exit     ;branch if X >= 38
                ;Y position >= 0 ?
                lda CURRENTYPOS
                cmp #$00
                bcc .plot_exit     ;branch if Y < 0
                ;Y position < 24 ?
                lda CURRENTYPOS
                cmp #SCREEN_HIGHT
                bcs .plot_exit     ;branch if Y >= 24
                ;get screen ram address
                jsr SetCharPos
                ;print the character
                lda CURRENTCHARACTER
                sta (SCREENL),Y
                ;get color ram address
                jsr SetColorPos
                ;print the color
                lda CURRENTCOLOR
                sta (SCREENL),Y
.plot_exit
                rts

;********************************************************************
; Subroutine:   ActivateScoreFlash
;********************************************************************
ActivateScoreFlash
                lda #SCCF
                sta scoreColor
                rts

;********************************************************************
; Subroutine:   DeactivateScoreFlash
;********************************************************************
DeactivateScoreFlash
                lda #SCCO
                sta scoreColor
                rts

;********************************************************************
; Subroutine:   ActivateHiScoreFlash
;********************************************************************
ActivateHiScoreFlash
                lda #SCCF
                sta hiScoreColor
                rts

;********************************************************************
; Subroutine:   DeactivateHiScoreFlash
;********************************************************************
DeactivateHiScoreFlash
                lda #SCCO
                sta hiScoreColor
                rts

;********************************************************************
;screen data ********************************************************
;********************************************************************
screenRAMRowLow     ;SCREENRAM + 40*0, 40*1, 40*2 ... 40*24
                !byte <SCREENRAM
                !byte <SCREENRAM+40
                !byte <SCREENRAM+80
                !byte <SCREENRAM+120
                !byte <SCREENRAM+160
                !byte <SCREENRAM+200
                !byte <SCREENRAM+240
                !byte <SCREENRAM+280
                !byte <SCREENRAM+320
                !byte <SCREENRAM+360
                !byte <SCREENRAM+400
                !byte <SCREENRAM+440
                !byte <SCREENRAM+480
                !byte <SCREENRAM+520
                !byte <SCREENRAM+560
                !byte <SCREENRAM+600
                !byte <SCREENRAM+640
                !byte <SCREENRAM+680
                !byte <SCREENRAM+720
                !byte <SCREENRAM+760
                !byte <SCREENRAM+800
                !byte <SCREENRAM+840
                !byte <SCREENRAM+880
                !byte <SCREENRAM+920
                !byte <SCREENRAM+960

screenRAMRowHigh    ;SCREENRAM + 40*0, 40*1, 40*2 ... 40*24
                !byte >SCREENRAM
                !byte >SCREENRAM+40
                !byte >SCREENRAM+80
                !byte >SCREENRAM+120
                !byte >SCREENRAM+160
                !byte >SCREENRAM+200
                !byte >SCREENRAM+240
                !byte >SCREENRAM+280
                !byte >SCREENRAM+320
                !byte >SCREENRAM+360
                !byte >SCREENRAM+400
                !byte >SCREENRAM+440
                !byte >SCREENRAM+480
                !byte >SCREENRAM+520
                !byte >SCREENRAM+560
                !byte >SCREENRAM+600
                !byte >SCREENRAM+640
                !byte >SCREENRAM+680
                !byte >SCREENRAM+720
                !byte >SCREENRAM+760
                !byte >SCREENRAM+800
                !byte >SCREENRAM+840
                !byte >SCREENRAM+880
                !byte >SCREENRAM+920
                !byte >SCREENRAM+960

colorRAMRowLow      ;COLORRAM + 40*0, 40*1, 40*2 ... 40*24
                !byte <COLORRAM
                !byte <COLORRAM+40
                !byte <COLORRAM+80
                !byte <COLORRAM+120
                !byte <COLORRAM+160
                !byte <COLORRAM+200
                !byte <COLORRAM+240
                !byte <COLORRAM+280
                !byte <COLORRAM+320
                !byte <COLORRAM+360
                !byte <COLORRAM+400
                !byte <COLORRAM+440
                !byte <COLORRAM+480
                !byte <COLORRAM+520
                !byte <COLORRAM+560
                !byte <COLORRAM+600
                !byte <COLORRAM+640
                !byte <COLORRAM+680
                !byte <COLORRAM+720
                !byte <COLORRAM+760
                !byte <COLORRAM+800
                !byte <COLORRAM+840
                !byte <COLORRAM+880
                !byte <COLORRAM+920
                !byte <COLORRAM+960

colorRAMRowHigh     ;COLORRAM + 40*0, 40*1, 40*2 ... 40*24
                !byte >COLORRAM
                !byte >COLORRAM+40
                !byte >COLORRAM+80
                !byte >COLORRAM+120
                !byte >COLORRAM+160
                !byte >COLORRAM+200
                !byte >COLORRAM+240
                !byte >COLORRAM+280
                !byte >COLORRAM+320
                !byte >COLORRAM+360
                !byte >COLORRAM+400
                !byte >COLORRAM+440
                !byte >COLORRAM+480
                !byte >COLORRAM+520
                !byte >COLORRAM+560
                !byte >COLORRAM+600
                !byte >COLORRAM+640
                !byte >COLORRAM+680
                !byte >COLORRAM+720
                !byte >COLORRAM+760
                !byte >COLORRAM+800
                !byte >COLORRAM+840
                !byte >COLORRAM+880
                !byte >COLORRAM+920
                !byte >COLORRAM+960

column          ;only to hold the column as double-byte number
                !byte SCREEN_WIDTH
                !byte 00

;********************************************************************
; SCREEN MODULE END                                                 *
;********************************************************************