;********************************************************************
; STARS MODULE                                                      *
;********************************************************************
;********************************************************************
; Subroutine:   StarsUpdate
;               Paralax effect using the characters:'CHR_STAR1 -
;               CHR_STAR8' with 4 different update speeds.
;               Called during raster interrupt in order to get smooth
;               and constant star movement, independently from the
;               current main runtime length.
;********************************************************************
StarsUpdate
                ;only when allowed
                lda STARSALLOWED
                cmp #TRUE
                beq .starsupdate_allowed
                rts
.starsupdate_allowed
                ldx #STARS_NUM_ROWS-1
.starsupdate_loop
                ;only update when the delay is zero
                dec starsDelayArray,X
                bpl .starsupdate_skipupdate
                ;reset the star delay
                lda starsSpeedColArray,X
                sta starsDelayArray,X
                ;get current screen address
                lda screenRAMRowLow,X
                sta STARSCREENL
                lda screenRAMRowHigh,X
                sta STARSCREENH
                ;move to the next frame
                dec starsFrameArray,X
                bpl .starsupdate_skip
                ;reset the star frame
                lda #$07
                sta starsFrameArray,X
                ;erase the current star
                ldy starsXCharArray,X
                lda (STARSCREENL),Y
                and #STARS_DEL_MASK
                cmp #CHR_STAR1
                bne .starsupdate_skiperase
                lda #CHR_SPACE
                sta (STARSCREENL),Y
.starsupdate_skiperase
                dec starsXCharArray,X
                bpl .starsupdate_skip
                ;reset char position
                lda #SCREEN_WIDTH-1
                sta starsXCharArray,X
                ;reset the speed & color
                inc STARSNEXTSPEED
                lda STARSNEXTSPEED
                and #$03
                sta starsSpeedColArray,X
.starsupdate_skip
                ;only draw the star if position is empty or star
                ldy starsXCharArray,X
                lda (STARSCREENL),Y
                cmp #CHR_SPACE
                beq .starsupdate_drawstar
                and #STARS_DEL_MASK
                cmp #CHR_STAR1
                bne .starsupdate_skipupdate
.starsupdate_drawstar
                ;draw the current star
                lda starsFrameArray,X
                eor #CHR_STAR1+7
                sta (STARSCREENL),Y
                lda STARSCREENH
                eor #>(SCREENRAM XOR COLORRAM)
                sta STARSCREENH
                ;convert the color to C16
                ldy starsSpeedColArray,X
                lda colorConversion,Y
                ldy starsXCharArray,X
                sta (STARSCREENL),Y
.starsupdate_skipupdate
                dex
                bpl .starsupdate_loop
                rts

;********************************************************************
;game stars data ****************************************************
;********************************************************************
starsXCharArray
                !byte  18, 26,  9, 23, 35, 14,  2, 20, 29,  8
                !byte  32,  4, 30, 11, 34, 17, 38,  6, 25, 13
                !byte  36, 19, 22, 39,  1, 28, 40, 15, 37, 33
                !byte  10, 27,  5, 16, 31, 12,  7, 24,  3, 21

starsFrameArray
                !fill STARS_NUM_ROWS, 0

starsDelayArray
                !fill STARS_NUM_ROWS, 1

starsSpeedColArray
                !byte  0,  3,  1,  0,  1,  2,  0,  1,  0,  2
                !byte  0,  2,  0,  3,  0,  2,  3,  1,  2,  0
                !byte  2,  1,  3,  1,  3,  1,  0,  2,  0,  1
                !byte  0,  1,  0,  1,  0,  2,  1,  0,  3,  2

colorConversion
                ;slow to fast
                !byte  WHT, YEL7, PUR7, CYN7

;********************************************************************
; STARS MODULE END                                                  *
;********************************************************************